@@ -78,9 +78,10 @@ void insert_accounting_structure(struct pkt_data *data)
     if (elem_acc) {
       if (timeval_cmp(&data->cst.stamp, &elem_acc->rstamp) >= 0 && 
 	  timeval_cmp(&data->cst.stamp, &table_reset_stamp) >= 0) {
-        elem_acc->bytes_counter -= data->cst.ba;
-        elem_acc->packet_counter -= data->cst.pa;
-        elem_acc->flow_counter -= data->cst.fa;
+	/* MIN(): ToS issue */
+        elem_acc->bytes_counter -= MIN(elem_acc->bytes_counter, data->cst.ba);
+        elem_acc->packet_counter -= MIN(elem_acc->packet_counter, data->cst.pa);
+        elem_acc->flow_counter -= MIN(elem_acc->flow_counter, data->cst.fa);
       } 
       else memset(&data->cst, 0, CSSz);
     }
