@@ -35,12 +35,14 @@ struct configuration {
   char *type;
   u_int32_t what_to_count;
   int acct_type; 
+  int data_type; 
   int pipe_size;
   int buffer_size;
   int handle_fragments;
   int handle_flows;
   int frag_bufsz;
   int flow_bufsz;
+  int flow_hashsz;
   int conntrack_bufsz;
   int flow_lifetime;
   char *imt_plugin_path;
@@ -58,11 +60,13 @@ struct configuration {
   int sql_refresh_time;
   int sql_history;
   int sql_history_howmany; /* internal */
+  int sql_history_since_epoch;
   int sql_startup_delay;
   int sql_cache_entries;
   int sql_dont_try_update;
   char *sql_history_roundoff;
   char *sql_recovery_logfile;
+  int sql_max_writers;
   int sql_trigger_time;
   int sql_trigger_time_howmany; /* internal */
   char *sql_trigger_exec;
@@ -71,6 +75,8 @@ struct configuration {
   int sql_preprocess_type;
   int sql_multi_values;
   int sql_aggressive_classification;
+  char *sql_locking_style;
+  int sql_use_copy;
   int print_refresh_time;
   int print_cache_entries;
   int print_markers;
@@ -91,6 +97,8 @@ struct configuration {
   int buckets;
   int daemon;
   int active_plugins;
+  char *logfile; 
+  FILE *logfile_fd; 
   char *pidfile; 
   int networks_mask;
   char *networks_file;
@@ -109,9 +117,21 @@ struct configuration {
   char *classifiers_path;
   int classifier_tentatives;
   int classifier_table_num;
+  u_int32_t nfprobe_what_to_count;
+  char *nfprobe_timeouts;
+  int nfprobe_id;
+  int nfprobe_hoplimit;
+  int nfprobe_maxflows;
+  char *nfprobe_receiver;
+  int nfprobe_version;
+  char *nfprobe_engine;
+  char *sfprobe_receiver;
+  char *sfprobe_agentip;
+  int sfprobe_agentsubid;
 };
 
 struct plugin_type_entry {
+  int id;
   char string[10];
   void (*func)(int, struct configuration *, void *);
 };
