@@ -28,7 +28,6 @@
 #include "ip_flow.h"
 #include "classifier.h"
 #include "jhash.h"
-#include "util.h"
 
 u_int32_t flt_total_nodes;  
 time_t flt_prune_deadline;
@@ -53,10 +52,16 @@ void init_ip_flow_handler()
 
 void init_ip4_flow_handler()
 {
+  int size;
+
   if (config.flow_bufsz) flt_total_nodes = config.flow_bufsz / sizeof(struct ip_flow);
   else flt_total_nodes = DEFAULT_FLOW_BUFFER_SIZE / sizeof(struct ip_flow); 
 
-  memset(ip_flow_table, 0, sizeof(ip_flow_table));
+  if (!config.flow_hashsz) config.flow_hashsz = FLOW_TABLE_HASHSZ; 
+  size = sizeof(struct ip_flow) * config.flow_hashsz;
+  ip_flow_table = (struct ip_flow **) malloc(size);
+
+  memset(ip_flow_table, 0, size);
   flow_lru_list.root = (struct ip_flow *) malloc(sizeof(struct ip_flow)); 
   flow_lru_list.last = flow_lru_list.root;
   memset(flow_lru_list.root, 0, sizeof(struct ip_flow));
@@ -346,7 +351,7 @@ unsigned int normalize_flow(u_int32_t *ip_src, u_int32_t *ip_dst,
 unsigned int hash_flow(u_int32_t ip_src, u_int32_t ip_dst,
 		u_int16_t port_src, u_int16_t port_dst, u_int8_t proto)
 {
-  return jhash_3words((u_int32_t)(port_src ^ port_dst) << 16 | proto, ip_src, ip_dst, flt_trivial_hash_rnd) & (FLOW_TABLE_HASHSZ-1);
+  return jhash_3words((u_int32_t)(port_src ^ port_dst) << 16 | proto, ip_src, ip_dst, flt_trivial_hash_rnd) & (config.flow_hashsz-1);
 }
 
 /* is_expired() checks for the expiration of the bi-directional flow; returns: TRUE if
@@ -389,10 +394,16 @@ unsigned int is_expired_uni(struct timeval *now, struct ip_flow_common *fp, unsi
 #if defined ENABLE_IPV6
 void init_ip6_flow_handler()
 {
+  int size;
+
   if (config.frag_bufsz) flt6_total_nodes = config.frag_bufsz / sizeof(struct ip_flow6);
   else flt6_total_nodes = DEFAULT_FLOW_BUFFER_SIZE / sizeof(struct ip_flow6);
 
-  memset(ip_flow_table6, 0, sizeof(ip_flow_table6));
+  if (!config.flow_hashsz) config.flow_hashsz = FLOW_TABLE_HASHSZ;
+  size = sizeof(struct ip_flow6) * config.flow_hashsz;
+  ip_flow_table6 = (struct ip_flow6 **) malloc(size);
+
+  memset(ip_flow_table6, 0, size);
   flow_lru_list6.root = (struct ip_flow6 *) malloc(sizeof(struct ip_flow6));
   flow_lru_list6.last = flow_lru_list6.root;
   memset(flow_lru_list6.root, 0, sizeof(struct ip_flow6));
@@ -445,7 +456,7 @@ unsigned int hash_flow6(u_int32_t id, struct in6_addr *saddr, struct in6_addr *d
         c += id;
         __jhash_mix(a, b, c);
 
-        return c & (FLOW_TABLE_HASHSZ - 1);
+        return c & (config.flow_hashsz - 1);
 }
 
 unsigned int normalize_flow6(struct in6_addr *saddr, struct in6_addr *daddr,
