@@ -30,7 +30,7 @@ void Log(short int level, char *msg, ...)
   
   if ((level == LOG_DEBUG) && !config.debug) return;
 
-  if (!config.syslog) {
+  if (!config.syslog && !config.logfile_fd) {
     va_start(ap, msg);
     vprintf(msg, ap);
     va_end(ap);
@@ -39,9 +39,16 @@ void Log(short int level, char *msg, ...)
     va_start(ap, msg);
     vsnprintf(syslog_string, LOGSTRLEN, msg, ap);
     va_end(ap);
-    sbrk(LOGSTRLEN);
-    syslog(level, syslog_string);
-    sbrk(-LOGSTRLEN);
+
+    if (config.syslog) {
+      sbrk(LOGSTRLEN);
+      syslog(level, syslog_string);
+      sbrk(-LOGSTRLEN);
+    }
+    if (config.logfile_fd) { 
+      fprintf(config.logfile_fd, syslog_string);
+      fflush(config.logfile_fd);
+    }
   }
 }
 
