@@ -27,7 +27,6 @@
 #include "plugin_hooks.h"
 #include "sql_common.h"
 #include "mysql_plugin.h"
-#include "util.h"
 #include "sql_common_m.c"
 
 /* Functions */
@@ -41,7 +40,6 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   time_t now, refresh_deadline;
   int timeout;
   int ret, num;
-#if defined (HAVE_MMAP)
   struct ring *rg = &((struct channels_list_entry *)ptr)->rg;
   struct ch_status *status = ((struct channels_list_entry *)ptr)->status;
   u_int32_t bufsz = ((struct channels_list_entry *)ptr)->bufsize;
@@ -49,7 +47,6 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   unsigned char *rgptr;
   int pollagain = TRUE;
   u_int32_t seq = 1, rg_err_count = 0; 
-#endif
 
   /* XXX: glue */
   memcpy(&config, cfgptr, sizeof(struct configuration));
@@ -99,9 +96,7 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   /* plugin main loop */
   for(;;) {
     poll_again:
-#if defined (HAVE_MMAP)
     status->wakeup = TRUE;
-#endif
     ret = poll(&pfd, 1, timeout);
     if (ret < 0) goto poll_again;
 
@@ -109,6 +104,7 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
     switch (ret) {
     case 0: /* timeout */
+      if (qq_ptr) sql_cache_flush(queries_queue, qq_ptr, &idata);
       switch (fork()) {
       case 0: /* Child */
 	/* we have to ignore signals to avoid loops:
@@ -117,7 +113,8 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	signal(SIGHUP, SIG_IGN);
 	pm_setproctitle("%s [%s]", "MySQL Plugin -- DB Writer", config.name);
 
-	if (qq_ptr) {
+        if (qq_ptr && sql_writers.flags != CHLD_ALERT) {
+	  if (sql_writers.flags == CHLD_WARNING) sql_db_fail(&p);
           (*sqlfunc_cbr.connect)(&p, config.sql_host); 
           (*sqlfunc_cbr.purge)(queries_queue, qq_ptr, &idata);
 	  (*sqlfunc_cbr.close)(&bed);
@@ -129,7 +126,6 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	  
         exit(0);
       default: /* Parent */
-	if (qq_ptr) qq_ptr = sql_cache_flush(queries_queue, qq_ptr);
 	gettimeofday(&idata.flushtime, &tz);
 	refresh_deadline += config.sql_refresh_time; 
 	if (idata.now > idata.triggertime) {
@@ -139,6 +135,9 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	}
 	idata.new_basetime = FALSE;
 	glob_new_basetime = FALSE;
+	qq_ptr = pqq_ptr;
+	memcpy(queries_queue, pending_queries_queue, sizeof(queries_queue));
+
 	if (reload_map) {
 	  load_networks(config.networks_file, &nt, &nc);
 	  load_ports(config.ports_file, &pt);
@@ -148,12 +147,6 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       }
       break;
     default: /* we received data */
-#if !defined (HAVE_MMAP)
-      if ((ret = read(pipe_fd, pipebuf, config.buffer_size)) == 0) 
-        exit_plugin(1); /* we exit silently; something happened at the write end */
-
-      if (ret < 0) goto poll_again;
-#else
       read_data:
       if (!pollagain) {
         seq++;
@@ -188,10 +181,10 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       memcpy(pipebuf, rg->ptr, bufsz);
       if ((rg->ptr+bufsz) >= rg->end) rg->ptr = rg->base;
       else rg->ptr += bufsz;
-#endif
 
       /* lazy sql refresh handling */ 
       if (idata.now > refresh_deadline) {
+        if (qq_ptr) sql_cache_flush(queries_queue, qq_ptr, &idata);
         switch (fork()) {
         case 0: /* Child */
           /* we have to ignore signals to avoid loops:
@@ -200,7 +193,8 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
           signal(SIGHUP, SIG_IGN);
 	  pm_setproctitle("%s [%s]", "MySQL Plugin -- DB Writer", config.name);
 
-	  if (qq_ptr) {
+          if (qq_ptr && sql_writers.flags != CHLD_ALERT) {
+	    if (sql_writers.flags == CHLD_WARNING) sql_db_fail(&p);
             (*sqlfunc_cbr.connect)(&p, config.sql_host);
             (*sqlfunc_cbr.purge)(queries_queue, qq_ptr, &idata);
 	    (*sqlfunc_cbr.close)(&bed);
@@ -212,7 +206,6 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
           exit(0);
         default: /* Parent */
-          if (qq_ptr) qq_ptr = sql_cache_flush(queries_queue, qq_ptr);
 	  gettimeofday(&idata.flushtime, &tz);
 	  refresh_deadline += config.sql_refresh_time; 
           if (idata.now > idata.triggertime) {
@@ -222,6 +215,9 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	  }
 	  idata.new_basetime = FALSE;
 	  glob_new_basetime = FALSE;
+	  qq_ptr = pqq_ptr;
+	  memcpy(queries_queue, pending_queries_queue, sizeof(queries_queue));
+
 	  if (reload_map) {
 	    load_networks(config.networks_file, &nt, &nc);
 	    load_ports(config.ports_file, &pt);
@@ -265,9 +261,7 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	((struct ch_buf_hdr *)pipebuf)->num--;
         if (((struct ch_buf_hdr *)pipebuf)->num) data++;
       }
-#if defined (HAVE_MMAP)
       goto read_data;
-#endif
     }
   }
 }
@@ -303,39 +297,6 @@ int MY_cache_dbop(struct DBdesc *db, struct db_cache *cache_elem, struct insert_
   
   /* sending UPDATE query */
   if (!config.sql_dont_try_update) {
-    /* searching for pending accumulators: this means we know of unclassified counters has been
-       kicked to the DB previously; we now try to remove such data by issuing a negative UPDATE
-       query. If we are successfull, we add the counters to the new class. Otherwise we discard
-       them. */
-    if (config.what_to_count & COUNT_CLASS && cache_elem->ba) {
-      char local_where_clause[LONGSRVBUFLEN], local_values_clause[LONGSRVBUFLEN];
-      char *local_ptr_where = local_where_clause, *local_ptr_values = local_values_clause;
-      pm_class_t tmp = cache_elem->primitives.class;
-
-      cache_elem->primitives.class = 0; num = 0;
-      memset(local_where_clause, 0, sizeof(local_where_clause));
-      memset(local_values_clause, 0, sizeof(local_values_clause));
-      while (num < idata->num_primitives) {
-	(*where[num].handler)(cache_elem, idata, num, &local_ptr_values, &local_ptr_where);
-	num++;
-      }
-      if (have_flows) ret = snprintf(sql_data, sizeof(sql_data), update_negative_clause, cache_elem->pa, cache_elem->ba, cache_elem->fa);
-      else ret = snprintf(sql_data, sizeof(sql_data), update_negative_clause, cache_elem->pa, cache_elem->ba);
-      strncpy(sql_data+ret, local_where_clause, SPACELEFT_LEN(sql_data, ret));
-      cache_elem->primitives.class = tmp;
-
-      ret = mysql_query(db->desc, sql_data);
-      if (ret) goto signal_error;
-      if (mysql_affected_rows(db->desc)) {
-	cache_elem->bytes_counter += cache_elem->ba;
-	cache_elem->packet_counter += cache_elem->pa;
-	cache_elem->flows_counter += cache_elem->fa;
-
-	Log(LOG_DEBUG, "DEBUG ( %s/%s ): %s\n\n", config.name, config.type, sql_data);
-        // idata->uqn++; /* XXX: negative UPDATE queries number ? */
-      }
-    }
-
     if (have_flows) ret = snprintf(sql_data, sizeof(sql_data), update_clause, cache_elem->packet_counter, cache_elem->bytes_counter, cache_elem->flows_counter);
     else ret = snprintf(sql_data, sizeof(sql_data), update_clause, cache_elem->packet_counter, cache_elem->bytes_counter);
     strncpy(sql_data+ret, where_clause, SPACELEFT_LEN(sql_data, ret));
@@ -408,7 +369,7 @@ int MY_cache_dbop(struct DBdesc *db, struct db_cache *cache_elem, struct insert_
   }
 
   idata->een++;
-  cache_elem->valid = FALSE; /* committed */
+  // cache_elem->valid = FALSE; /* committed */
 
   return ret;
 
@@ -454,13 +415,10 @@ void MY_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
 
     strftime_same(insert_clause, LONGSRVBUFLEN, tmpbuf, &idata->basetime);
     strftime_same(update_clause, LONGSRVBUFLEN, tmpbuf, &idata->basetime);
-    strftime_same(update_negative_clause, LONGSRVBUFLEN, tmpbuf, &idata->basetime);
     strftime_same(lock_clause, LONGSRVBUFLEN, tmpbuf, &idata->basetime);
-    strftime_same(delete_shadows_clause, LONGSRVBUFLEN, tmpbuf, &idata->basetime);
     if (config.sql_table_schema && idata->new_basetime) sql_create_table(bed.p, idata);
   }
   strncat(update_clause, set_clause, SPACELEFT(update_clause));
-  strncat(update_negative_clause, set_negative_clause, SPACELEFT(update_negative_clause));
 
   (*sqlfunc_cbr.lock)(bed.p); 
 
@@ -471,13 +429,10 @@ void MY_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
   /* multi-value INSERT query: wrap-up */
   if (idata->mv.buffer_elem_num) {
     idata->mv.last_queue_elem = TRUE;
-    queue[idata->current_queue_elem-1]->valid = TRUE;
+    queue[idata->current_queue_elem-1]->valid = SQL_CACHE_COMMITTED;
     sql_query(&bed, queue[idata->current_queue_elem-1], idata);
   }
 
-  if (config.what_to_count & COUNT_CLASS && !config.sql_dont_try_update)
-    (*sqlfunc_cbr.delete_shadows)(&bed);
-
   /* rewinding stuff */
   (*sqlfunc_cbr.unlock)(&bed);
   if ((lf.fail) || (b.fail)) Log(LOG_ALERT, "ALERT ( %s/%s ): recovery for MySQL daemon failed.\n", config.name, config.type);
@@ -485,7 +440,7 @@ void MY_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
   if (config.debug) {
     idata->elap_time = time(NULL)-start; 
     Log(LOG_DEBUG, "( %s/%s ) *** Purging cache - END (QN: %u, ET: %u) ***\n", 
-		    config.name, config.type, index, idata->elap_time); 
+		    config.name, config.type, idata->qn, idata->elap_time); 
   }
 
   if (config.sql_trigger_exec) {
@@ -502,11 +457,17 @@ int MY_evaluate_history(int primitive)
       strncat(values[primitive].string, ", ", sizeof(values[primitive].string));
       strncat(where[primitive].string, " AND ", sizeof(where[primitive].string));
     }
-    strncat(where[primitive].string, "FROM_UNIXTIME(%u) = ", SPACELEFT(where[primitive].string));
+    if (!config.sql_history_since_epoch)
+      strncat(where[primitive].string, "FROM_UNIXTIME(%u) = ", SPACELEFT(where[primitive].string));
+    else
+      strncat(where[primitive].string, "%u = ", SPACELEFT(where[primitive].string));
     strncat(where[primitive].string, "stamp_inserted", SPACELEFT(where[primitive].string));
 
     strncat(insert_clause, "stamp_updated, stamp_inserted", SPACELEFT(insert_clause));
-    strncat(values[primitive].string, "FROM_UNIXTIME(%u), FROM_UNIXTIME(%u)", SPACELEFT(values[primitive].string));
+    if (!config.sql_history_since_epoch)
+      strncat(values[primitive].string, "FROM_UNIXTIME(%u), FROM_UNIXTIME(%u)", SPACELEFT(values[primitive].string));
+    else
+      strncat(values[primitive].string, "%u, %u", SPACELEFT(values[primitive].string));
 
     where[primitive].type = values[primitive].type = TIMESTAMP;
     values[primitive].handler = where[primitive].handler = count_timestamp_handler;
@@ -541,12 +502,12 @@ int MY_compose_static_queries()
   strncat(insert_clause, ")", SPACELEFT(insert_clause));
 
   /* "LOCK ..." stuff */
+  if (config.sql_locking_style) Log(LOG_WARNING, "WARN ( %s/%s ): sql_locking_style is not supported. Ignored.\n", config.name, config.type);
   snprintf(lock_clause, sizeof(lock_clause), "LOCK TABLES %s WRITE", config.sql_table);
   strncpy(unlock_clause, "UNLOCK TABLES", sizeof(unlock_clause));
 
   /* "UPDATE ... SET ..." stuff */
   snprintf(update_clause, sizeof(update_clause), "UPDATE %s ", config.sql_table);
-  snprintf(update_negative_clause, sizeof(update_negative_clause), "UPDATE %s ", config.sql_table);
 #if defined HAVE_64BIT_COUNTERS
   strncpy(set_clause, "SET packets=packets+%llu, bytes=bytes+%llu", SPACELEFT(set_clause));
   if (have_flows) strncat(set_clause, ", flows=flows+%llu", SPACELEFT(set_clause));
@@ -554,33 +515,16 @@ int MY_compose_static_queries()
   strncpy(set_clause, "SET packets=packets+%lu, bytes=bytes+%lu", SPACELEFT(set_clause));
   if (have_flows) strncat(set_clause, ", flows=flows+%lu", SPACELEFT(set_clause));
 #endif
-  if (config.sql_history) strncat(set_clause, ", stamp_updated=now()", SPACELEFT(set_clause));
-  strncpy(set_negative_clause, "SET packets=packets-%lu, bytes=bytes-%lu", SPACELEFT(set_negative_clause));
-  if (have_flows) strncat(set_negative_clause, ", flows=flows-%lu", SPACELEFT(set_negative_clause));
-  if (config.sql_history) strncat(set_negative_clause, ", stamp_updated=now()", SPACELEFT(set_negative_clause));
-
-  /* "DELETE ..." stuff */
-  snprintf(delete_shadows_clause, sizeof(delete_shadows_clause), "DELETE FROM %s WHERE packets = 0 AND bytes = 0 AND flows = 0", config.sql_table);
+  if (config.sql_history) {
+    if (!config.sql_history_since_epoch)
+      strncat(set_clause, ", stamp_updated=NOW()", SPACELEFT(set_clause));
+    else
+      strncat(set_clause, ", stamp_updated=UNIX_TIMESTAMP(NOW())", SPACELEFT(set_clause));
+  }
 
   return primitives;
 }
 
-void MY_delete_shadows(struct BE_descs *bed)
-{
-  struct DBdesc *db = NULL; 
-
-  if (bed->p->connected) db = bed->p;
-  else if (bed->b->connected) db = bed->b;
-
-  if (!db->fail) {
-    if (mysql_query(db->desc, delete_shadows_clause)) {
-      MY_get_errmsg(db);
-      sql_db_errmsg(db);
-      sql_db_fail(db);
-    }
-  }
-}
-
 void MY_Lock(struct DBdesc *db)
 {
   if (!db->fail) {
@@ -612,14 +556,16 @@ void MY_DB_Connect(struct DBdesc *db, char *host)
 {
   MYSQL *dbptr = db->desc;
 
-  mysql_init(db->desc);
-  dbptr->reconnect = TRUE;
-  if (!mysql_real_connect(db->desc, host, config.sql_user, config.sql_passwd, config.sql_db, 0, NULL, 0)) {
-    sql_db_fail(db);
-    MY_get_errmsg(db);
-    sql_db_errmsg(db);
+  if (!db->fail) {
+    mysql_init(db->desc);
+    dbptr->reconnect = TRUE;
+    if (!mysql_real_connect(db->desc, host, config.sql_user, config.sql_passwd, config.sql_db, 0, NULL, 0)) {
+      sql_db_fail(db);
+      MY_get_errmsg(db);
+      sql_db_errmsg(db);
+    }
+    else sql_db_ok(db);
   }
-  else sql_db_ok(db);
 }
 
 void MY_DB_Close(struct BE_descs *bed)
@@ -630,10 +576,12 @@ void MY_DB_Close(struct BE_descs *bed)
 
 void MY_create_dyn_table(struct DBdesc *db, char *buf)
 {
-  if (mysql_query(db->desc, buf)) {
-    Log(LOG_DEBUG, "DEBUG ( %s/%s ): FAILED query follows:\n%s\n", config.name, config.type, buf);
-    MY_get_errmsg(db);
-    sql_db_errmsg(db);
+  if (!db->fail) {
+    if (mysql_query(db->desc, buf)) {
+      Log(LOG_DEBUG, "DEBUG ( %s/%s ): FAILED query follows:\n%s\n", config.name, config.type, buf);
+      MY_get_errmsg(db);
+      sql_db_errmsg(db);
+    }
   }
 }
 
@@ -656,7 +604,6 @@ void MY_set_callbacks(struct sqlfunc_cb_registry *cbr)
   cbr->close = MY_DB_Close;
   cbr->lock = MY_Lock;
   cbr->unlock = MY_Unlock;
-  cbr->delete_shadows = MY_delete_shadows;
   cbr->op = MY_cache_dbop;
   cbr->create_table = MY_create_dyn_table;
   cbr->purge = MY_cache_purge;
