@@ -25,6 +25,7 @@
 /* prototypes */
 void pgsql_plugin(int, struct configuration *, void *);
 int PG_cache_dbop(struct DBdesc *, struct db_cache *, struct insert_data *);
+int PG_cache_dbop_copy(struct DBdesc *, struct db_cache *, struct insert_data *);
 void PG_cache_purge(struct db_cache *[], int, struct insert_data *);
 int PG_evaluate_history(int);
 int PG_compose_static_queries();
@@ -33,7 +34,6 @@ void PG_Lock(struct DBdesc *);
 void PG_file_close(struct logfile *);
 void PG_DB_Connect(struct DBdesc *, char *);
 void PG_DB_Close(struct BE_descs *);
-void PG_delete_shadows(struct BE_descs *);
 void PG_create_dyn_table(struct DBdesc *, char *);
 static int PG_affected_rows(PGresult *);
 void PG_create_backend(struct DBdesc *);
