@@ -23,6 +23,7 @@
 
 /* includes */
 #include "pmacct.h"
+#include "pmacct-data.h"
 #include "plugin_hooks.h"
 #include "pkt_handlers.h"
 
@@ -37,7 +38,7 @@
    area */
 void load_plugins(struct plugin_requests *req)
 {
-  int x, v, socklen;
+  int x, v, socklen, nfprobe_id = 0, min_sz = 0;
   struct plugins_list_entry *list = plugins_list;
   int l = sizeof(list->cfg.pipe_size);
   struct channels_list_entry *chptr = NULL;
@@ -47,22 +48,29 @@ void load_plugins(struct plugin_requests *req)
 
   while (list) {
     if ((*list->type.func)) {
-#if defined (HAVE_MMAP)
-    /* If nothing is supplied, let's hint some working default values */
-    if (list->cfg.pcap_savefile && !list->cfg.pipe_size && !list->cfg.buffer_size) {
-      list->cfg.pipe_size = 4096000; /* 4Mb */
-      list->cfg.buffer_size = 10240; /* 10Kb */
-    }
-#endif
+      if (list->cfg.data_type & (PIPE_TYPE_METADATA|PIPE_TYPE_PAYLOAD));
+      else {
+	Log(LOG_ERR, "ERROR ( %s/%s ): Data type not supported: %d\n", list->name, list->type.string, list->cfg.data_type);
+	exit(1);
+      }
+
+      min_sz = ChBufHdrSz;
+      if (list->cfg.data_type & PIPE_TYPE_METADATA) min_sz += PdataSz; 
+      if (list->cfg.data_type & PIPE_TYPE_PAYLOAD) {
+	if (list->cfg.acct_type == ACCT_PM && list->cfg.snaplen) min_sz += (PpayloadSz+list->cfg.snaplen); 
+	else min_sz += (PpayloadSz+DEFAULT_PLOAD_SIZE); 
+      }
+      if (list->cfg.data_type & PIPE_TYPE_EXTRAS) min_sz += PextrasSz; 
+      
+      /* If nothing is supplied, let's hint some working default values */
+      if (list->cfg.pcap_savefile && !list->cfg.pipe_size && !list->cfg.buffer_size) {
+        list->cfg.pipe_size = 4096000; /* 4Mb */
+        list->cfg.buffer_size = 10240; /* 10Kb */
+      }
       /* creating communication channel */
       socketpair(AF_UNIX, SOCK_DGRAM, 0, list->pipe);
       if (list->cfg.pipe_size) {
-	if (list->cfg.pipe_size < (sizeof(struct pkt_data)+sizeof(struct ch_buf_hdr)))
-	  list->cfg.pipe_size = sizeof(struct pkt_data)+sizeof(struct ch_buf_hdr);
-#if !defined (HAVE_MMAP)
-	Setsocksize(list->pipe[0], SOL_SOCKET, SO_RCVBUF, &list->cfg.pipe_size, l);
-	Setsocksize(list->pipe[1], SOL_SOCKET, SO_SNDBUF, &list->cfg.pipe_size, l);
-#endif
+	if (list->cfg.pipe_size < min_sz) list->cfg.pipe_size = min_sz;
       }
       else {
         x = DEFAULT_PIPE_SIZE;
@@ -76,19 +84,18 @@ void load_plugins(struct plugin_requests *req)
       getsockopt(list->pipe[1], SOL_SOCKET, SO_SNDBUF, &v, &l);
       socklen = (v < x) ? v : x;
 
-#if !defined (HAVE_MMAP)
-      if ((socklen < list->cfg.pipe_size) || (list->cfg.debug)) 
-	Log(LOG_INFO, "INFO ( %s/%s ): Pipe size obtained: %d / %d.\n", 
-		list->name, list->type.string, socklen, list->cfg.pipe_size);
+      /* checking Core <-> Plugins buffer size; then, if required let's align it on
+         4 bytes boundary -- on the assumption that data strucures are aligned aswell. */
+      if (list->cfg.buffer_size < min_sz) list->cfg.buffer_size = min_sz;
+#if NEED_ALIGN
+      while (list->cfg.buffer_size % 4 != 0) list->cfg.buffer_size--;
 #endif
 
-      /* checking transfer buffer size between 'core' <-> 'plugins' */
-      if (list->cfg.buffer_size < (sizeof(struct pkt_data)+sizeof(struct ch_buf_hdr)))
-        list->cfg.buffer_size = sizeof(struct pkt_data)+sizeof(struct ch_buf_hdr);
-#if !defined (HAVE_MMAP)
-      if (list->cfg.buffer_size > socklen)
-        list->cfg.buffer_size = socklen; 
-#else
+      if (list->cfg.data_type == PIPE_TYPE_PAYLOAD) {
+	/* Let's tweak plugin_pipe_size if we don't have an explicit size */
+	if (!list->cfg.pipe_size) list->cfg.pipe_size = 4096000; /* 4Mb */
+      }
+
       /* if we are not supplied a 'plugin_pipe_size', then we calculate it using
          buffer size and given socket size; if 'plugin_pipe_size' is known, we
 	 reverse the method: we try to obtain needed socket size to accomodate
@@ -121,20 +128,6 @@ void load_plugins(struct plugin_requests *req)
         if ((x < socklen) || (list->cfg.debug))
           Log(LOG_INFO, "INFO ( %s/%s ): Pipe size obtained: %d / %d.\n", list->name, list->type.string, x, socklen);
       }
-#endif
-
-      /* compiling aggregation filter if needed */
-      if (list->cfg.a_filter) {
-	pcap_t *dev_desc = pcap_open_dead(1, 128); /* 128 bytes should be long enough */
-	bpf_u_int32 localnet, netmask;  /* pcap library stuff */
-	char errbuf[PCAP_ERRBUF_SIZE];
-
-	pcap_lookupnet(config.dev, &localnet, &netmask, errbuf);
-	if (pcap_compile(dev_desc, &list->cfg.bpfp_a_filter, list->cfg.a_filter, 0, netmask) < 0)
-          Log(LOG_WARNING, "WARN: %s\nWARN ( %s/%s ): aggregation filter disabled.\n", 
-			  pcap_geterr(dev_desc), config.name, config.type);
-	pcap_close(dev_desc);
-      }
 
       list->cfg.name = list->name;
       list->cfg.type = list->type.string;
@@ -143,12 +136,24 @@ void load_plugins(struct plugin_requests *req)
 	Log(LOG_ERR, "ERROR: Unable to setup a new Core Process <-> Plugin channel.\nExiting.\n"); 
 	exit_all(1);
       }
+      else chptr->plugin = list;
 
-#if defined (HAVE_MMAP)
-      if (!memcmp(list->type.string, "memory", strlen(list->type.string))) // || !memcmp(list->type.string, "print", strlen(list->type.string)))
-	chptr->request = TRUE; /* sets new value to be assigned to 'wakeup'; 'TRUE' disables on-request wakeup */ 
-#endif
+      /* sets new value to be assigned to 'wakeup'; 'TRUE' disables on-request wakeup */ 
+      if (list->type.id == PLUGIN_ID_MEMORY) chptr->request = TRUE; 
 
+      /* sets cleaner routine; XXX: we should definitely refine the way it works, maybe
+         by looking at stacking more of them, ie. extras assumes it's automagically piled
+	 with metadata */
+      if (list->cfg.data_type & PIPE_TYPE_METADATA) chptr->clean_func = pkt_data_clean;
+      if (list->cfg.data_type & PIPE_TYPE_PAYLOAD) chptr->clean_func = pkt_payload_clean;
+      if (list->cfg.data_type & PIPE_TYPE_EXTRAS) chptr->clean_func = pkt_extras_clean;
+
+      /* sets nfprobe ID */
+      if (list->type.id == PLUGIN_ID_NFPROBE) {
+	list->cfg.nfprobe_id = nfprobe_id;
+	nfprobe_id++;
+      }
+      
       switch (list->pid = fork()) {  
       case 0: /* Child */
 	/* SIGCHLD handling issue: SysV avoids zombies by ignoring SIGCHLD; to emulate
@@ -178,8 +183,8 @@ void load_plugins(struct plugin_requests *req)
 
 void exec_plugins(struct packet_ptrs *pptrs) 
 {
-  int num;
-  struct pkt_data *pdata;
+  int num, size, already_reprocessed = 0;
+  u_int32_t savedptr;
   char *bptr;
   int index;
 
@@ -187,42 +192,50 @@ void exec_plugins(struct packet_ptrs *pptrs)
     if (bpf_filter(channels_list[index].filter->bf_insns, pptrs->packet_ptr, pptrs->pkthdr->len,
         pptrs->pkthdr->caplen) && (!channels_list[index].tag_filter.num ||
 	!evaluate_tags(&channels_list[index].tag_filter, pptrs->tag)) && 
-	!evaluate_sampling(&channels_list[index].s)) {
-      /* constructing buffer: supported primitives + packet total length */
+	pptrs->tag_dist && // !evaluate_sampling(&channels_list[index].s) &&
+	!check_shadow_status(pptrs, &channels_list[index])) {
+      /* arranging buffer: supported primitives + packet total length */
+reprocess:
+      channels_list[index].reprocess = FALSE;
       num = 0;
-#if !defined (HAVE_MMAP)
-      pdata = (struct pkt_data *) channels_list[index].bufptr;
-#else
+
       /* rg.ptr points to slot's base address into the ring (shared memory); bufptr works
 	 as a displacement into the slot to place sequentially packets */
-      /* bufptr -> bptr -> pdata: avoids lvalue crap. Signalled by Andreas Jochens on AMD64/gcc4.0 */
       bptr = channels_list[index].rg.ptr+ChBufHdrSz+channels_list[index].bufptr; 
-      pdata = (struct pkt_data *)bptr; 
-      memset(pdata, 0, PdataSz);
-#endif
+      size = (*channels_list[index].clean_func)(bptr);
+      savedptr = channels_list[index].bufptr;
+      
       while (channels_list[index].phandler[num]) {
-        (*channels_list[index].phandler[num])(&channels_list[index], pptrs, pdata);
+        (*channels_list[index].phandler[num])(&channels_list[index], pptrs, &bptr);
         num++;
       }
 
-#if !defined (HAVE_MMAP)
-      ((struct ch_buf_hdr *)channels_list[index].buf)->num++;
-#else
-      channels_list[index].hdr.num++;
-#endif
-      channels_list[index].bufptr += PdataSz;
+      if (channels_list[index].s.rate && !channels_list[index].s.sampled_pkts) {
+	channels_list[index].reprocess = FALSE;
+	channels_list[index].bufptr = savedptr;
+	channels_list[index].hdr.num--; /* let's cheat this value as it will get increased later */
+	size = 0;
+      }
 
-      if ((channels_list[index].bufptr+PdataSz) > channels_list[index].bufend) {
-#if !defined (HAVE_MMAP)
-        if (write(channels_list[index].pipe, channels_list[index].buf, channels_list[index].bufsize) == -1) {
-	  if ((errno == EAGAIN) || (errno == ENOBUFS))
-	    Log(LOG_ERR, "ERROR: Pipe full. Raise maximum socket size for your system or try with a larger 'plugin_buffer_size' value. We are missing data.\n");
+      if (channels_list[index].reprocess) {
+        /* Let's check if we have an issue with the buffer size */
+        if (already_reprocessed) {
+          struct plugins_list_entry *list = channels_list[index].plugin;
+
+          Log(LOG_ERR, "ERROR ( %s/%s ): plugin_buffer_size is too short.\n", list->name, list->type.string);
+          exit_all(1);
         }
+        already_reprocessed = TRUE;
 
-	/* rewind pointer */
-        channels_list[index].bufptr = channels_list[index].buf+ChBufHdrSz;
-        ((struct ch_buf_hdr *)channels_list[index].buf)->num = 0;
-#else
+	/* Let's cheat the size in order to send out the current buffer */
+	size = channels_list[index].plugin->cfg.pipe_size;
+      }
+      else {
+        channels_list[index].hdr.num++;
+        channels_list[index].bufptr += size;
+      }
+
+      if ((channels_list[index].bufptr+size) > channels_list[index].bufend) {
 	channels_list[index].hdr.seq++;
 	channels_list[index].hdr.seq %= MAX_SEQNUM;
 
@@ -241,7 +254,8 @@ void exec_plugins(struct packet_ptrs *pptrs)
         /* rewind pointer */
         channels_list[index].bufptr = channels_list[index].buf;
         channels_list[index].hdr.num = 0;
-#endif
+
+	if (channels_list[index].reprocess) goto reprocess;
       }
     }
   }
@@ -260,19 +274,13 @@ struct channels_list_entry *insert_pipe_channel(struct configuration *cfg, int p
       chptr->filter = &cfg->bpfp_a_filter; 
       chptr->bufsize = cfg->buffer_size;
       chptr->id = cfg->post_tag;
-      if (cfg->sampling_rate) chptr->s.rate = cfg->sampling_rate;
-      for (x = 0; x < cfg->ptf.num; x++) chptr->tag_filter.table[x] = cfg->ptf.table[x];
-      chptr->tag_filter.num = cfg->ptf.num;
-#if !defined (HAVE_MMAP)
-      chptr->buf = malloc(cfg->buffer_size);
-      if (!chptr->buf) {
-        Log(LOG_ERR, "ERROR ( %s/%s ): unable to allocate channel buffer. Exiting ...\n", cfg->name, cfg->type);
-        exit_all(1);
+      if (cfg->sampling_rate) {
+	chptr->s.rate = cfg->sampling_rate;
+
+	if (cfg->acct_type == ACCT_NF) chptr->s.sf = &take_simple_systematic_skip;
+	else chptr->s.sf = &take_simple_random_skip;
       } 
-      memset(chptr->buf, 0, cfg->buffer_size);
-      chptr->bufptr = chptr->buf+sizeof(struct ch_buf_hdr);
-      chptr->bufend = chptr->buf+cfg->buffer_size;
-#else
+      memcpy(&chptr->tag_filter, &cfg->ptf, sizeof(struct pretag_filter));
       chptr->buf = 0;
       chptr->bufptr = chptr->buf;
       chptr->bufend = cfg->buffer_size-sizeof(struct ch_buf_hdr);
@@ -295,7 +303,6 @@ struct channels_list_entry *insert_pipe_channel(struct configuration *cfg, int p
         exit_all(1);
       }
       memset(chptr->status, 0, sizeof(struct ch_status));
-#endif
 
       break;
     }
@@ -384,36 +391,67 @@ void init_pipe_channels()
   memset(&channels_list, 0, MAX_N_PLUGINS*sizeof(struct channels_list_entry)); 
 }
 
-/* return value:
-   FALSE: take this packet into sample 
-   TRUE: discard this packet
-*/ 
-int evaluate_sampling(struct sampling *s)
+void evaluate_sampling(struct sampling *smp, pm_counter_t *pkt_len, pm_counter_t *pkt_num, pm_counter_t *sample_pool)
 {
-  if (!s->rate) return FALSE; /* sampling is disabled */
-  
-  if (s->counter) {
-    s->counter--; 
-    return TRUE;
-  }
-  else {
-    /* simple systematic algorithm */
-    // s->counter = s->rate;
+  pm_counter_t delta, pkts = *pkt_num;
 
-    /* simple random algorithm */
-    if (s->rate > 1) {
-      s->counter = ((random() % ((2 * s->rate) - 1)) + 1);
-      srandom(random());
-    }
-    else s->counter = 1; /* s->rate == 1 */
-
-    return FALSE;
+  if (!smp->rate) { /* sampling is disabled */
+    smp->sample_pool = pkts;
+    smp->sampled_pkts = pkts;
+    return;
   }
+
+  smp->sampled_pkts = 0;
+
+run_again: 
+  if (!smp->counter) smp->counter = (smp->sf)(smp->rate);
+
+  delta = MIN(smp->counter, pkts);
+  smp->counter -= delta;
+  pkts -= delta; 
+  smp->sample_pool += delta;
+
+  if (!smp->counter) {
+    smp->sampled_pkts++;
+    *sample_pool = smp->sample_pool;
+    smp->sample_pool = 0;
+    if (pkts > 0) goto run_again;
+  }
+
+  /* Let's handle flows meaningfully */
+  if (smp->sampled_pkts && *pkt_num > 1) {
+    *pkt_len = ( *pkt_len / *pkt_num ) * smp->sampled_pkts;
+    *pkt_num = smp->sampled_pkts;
+  }
+
+  // printf("RATE: %d, COUNTER: %d, PACKETS %d, SAMPLED_PACKETS: %d, SAMPLE_POOL: %d\n", smp->rate, smp->counter, *pkt_num, smp->sampled_pkts, *sample_pool);
+}
+
+/* simple random algorithm */
+pm_counter_t take_simple_random_skip(pm_counter_t mean)
+{
+  pm_counter_t skip;
+
+  if (mean > 1) {
+    skip = ((random() % ((2 * mean) - 1)) + 1);
+    srandom(random());
+  }
+  else skip = 1; /* smp->rate == 1 */
+
+  return skip;
+}
+
+/* simple systematic algorithm */
+pm_counter_t take_simple_systematic_skip(pm_counter_t mean)
+{
+  pm_counter_t skip = mean;
+
+  return skip;
 }
 
 /* return value:
-   FALSE: packet matched the filter
-   TRUE: discard this packet
+   TRUE: discard the packet/flow
+   FALSE: Go on with the packet/flow either because a) we had a match or b) we are not filtering 
 */
 int evaluate_tags(struct pretag_filter *filter, u_int16_t tag)
 {
@@ -422,9 +460,10 @@ int evaluate_tags(struct pretag_filter *filter, u_int16_t tag)
   if (filter->num == 0) return FALSE; /* no entries in the filter array: tag filtering disabled */
   
   for (index = 0; index < filter->num; index++) {
-    if (filter->table[index] == tag) return FALSE;
+    if (filter->table[index].n == tag) return (FALSE | filter->table[index].neg);
+    else if (filter->table[index].neg) return FALSE;
   }
-
+  
   return TRUE;
 }
 
@@ -435,12 +474,10 @@ void recollect_pipe_memory(struct channels_list_entry *mychptr)
 
   while (index < MAX_N_PLUGINS) {
     chptr = &channels_list[index];
-#if defined (HAVE_MMAP)
     if (mychptr->rg.base != chptr->rg.base) {
       munmap(chptr->rg.base, (chptr->rg.end-chptr->rg.base)+1550);
       munmap(chptr->status, sizeof(struct ch_status));
     }
-#endif
     index++;
   }
 }
@@ -458,9 +495,6 @@ void fill_pipe_buffer()
   int index;
 
   for (index = 0; channels_list[index].aggregation; index++) {
-#if !defined (HAVE_MMAP)
-    write(channels_list[index].pipe, channels_list[index].buf, channels_list[index].bufsize); 
-#else
     channels_list[index].hdr.seq++;
     channels_list[index].hdr.seq %= MAX_SEQNUM;
 
@@ -471,6 +505,64 @@ void fill_pipe_buffer()
       channels_list[index].status->wakeup = channels_list[index].request;
       write(channels_list[index].pipe, &channels_list[index].rg.ptr, CharPtrSz);
     }
-#endif
   }
 }
+
+int check_shadow_status(struct packet_ptrs *pptrs, struct channels_list_entry *mychptr)
+{
+  if (pptrs->shadow) {
+    if (pptrs->tag) {
+      if (mychptr->aggregation & COUNT_ID) return FALSE;
+      else return TRUE;
+    }
+    else TRUE;
+  } 
+  else return FALSE;
+}
+
+void load_plugin_filters(int link_type)
+{
+  struct plugins_list_entry *list = plugins_list;
+
+  while (list) {
+    if ((*list->type.func)) {
+
+      /* compiling aggregation filter if needed */
+      if (list->cfg.a_filter) {
+	pcap_t *dev_desc;
+	bpf_u_int32 localnet, netmask;  /* pcap library stuff */
+	char errbuf[PCAP_ERRBUF_SIZE];
+
+	dev_desc = pcap_open_dead(link_type, 128); /* 128 bytes should be long enough */
+
+	pcap_lookupnet(config.dev, &localnet, &netmask, errbuf);
+	if (pcap_compile(dev_desc, &list->cfg.bpfp_a_filter, list->cfg.a_filter, 0, netmask) < 0)
+	Log(LOG_WARNING, "WARN: %s\nWARN ( %s/%s ): aggregation filter disabled.\n",
+				pcap_geterr(dev_desc), config.name, config.type);
+	pcap_close(dev_desc);
+      }
+    }
+    list = list->next;
+  }
+}
+
+int pkt_data_clean(void *pdata)
+{
+  memset(pdata, 0, PdataSz);
+
+  return PdataSz;
+}
+
+int pkt_payload_clean(void *ppayload)
+{
+  memset(ppayload, 0, PpayloadSz);
+
+  return PpayloadSz;
+}
+
+int pkt_extras_clean(void *pextras)
+{
+  memset(pextras, 0, PdataSz+PextrasSz);
+
+  return PdataSz+PextrasSz;
+}
