@@ -44,6 +44,7 @@ int CHECK_Q_TYPE(int type)
 
   if (type & WANT_RESET) type ^= WANT_RESET;
   if (type & WANT_ERASE) type ^= WANT_ERASE;
+  if (type & WANT_LOCK_OP) type ^= WANT_LOCK_OP;
 
   return type;
 }
@@ -62,6 +63,7 @@ void usage_client(char *prog)
   printf("  -c\t[ src_mac | dst_mac | vlan | src_host | dst_host | src_port | dst_port | tos | proto | \n\t src_as | dst_as | sum_mac | sum_host | sum_net | sum_as | sum_port | tag | flows | \n\t class ] \n\tSelect primitives to match (required by -N and -M)\n");
   printf("  -e\tClear statistics\n");
   printf("  -r\tReset counters (applies to -N and -M)\n");
+  printf("  -l\tPerform locking of the table\n");
   printf("  -t\tShow memory table status\n");
   printf("  -C\tShow classifiers table\n");
   printf("  -p\t[file] \n\tSocket for client-server communication (DEFAULT: /tmp/collect.pipe)\n");
@@ -256,6 +258,7 @@ int main(int argc,char **argv)
   strcpy(path, "/tmp/collect.pipe");
   unpacked = 0; printed = 0;
   errflag = 0; buflen = 0;
+  protocols_number = 0;
   want_stats = FALSE;
   want_erase = FALSE;
   want_status = FALSE;
@@ -394,6 +397,9 @@ int main(int argc,char **argv)
       q.type |= WANT_STATUS; 
       want_status = TRUE;
       break;
+    case 'l':
+      q.type |= WANT_LOCK_OP;
+      break;
     case 'm': /* obsoleted */
       want_mrtg = TRUE;
     case 'N':
@@ -478,6 +484,9 @@ int main(int argc,char **argv)
   }
 
   memcpy(q.passwd, password, sizeof(password));
+
+  /* setting number of entries in _protocols structure */
+  while (_protocols[protocols_number].number != -1) protocols_number++;
   
   if (want_counter || want_match) {
     FILE *f;
@@ -827,8 +836,11 @@ int main(int argc,char **argv)
 	if (!have_wtc || (what_to_count & COUNT_DST_PORT))
 	  printf("%-5d     ", acc_elem->primitives.dst_port);
 
-	if (!have_wtc || (what_to_count & COUNT_IP_PROTO))
-	  printf("%-10s  ", _protocols[acc_elem->primitives.proto].name);
+	if (!have_wtc || (what_to_count & COUNT_IP_PROTO)) {
+	  if (acc_elem->primitives.proto < protocols_number)
+	    printf("%-10s  ", _protocols[acc_elem->primitives.proto].name);
+	  else printf("%-3d  ", acc_elem->primitives.proto);
+	}
 
 	if (!have_wtc || (what_to_count & COUNT_IP_TOS))
 	  printf("%-3d    ", acc_elem->primitives.tos); 
