@@ -157,6 +157,7 @@ struct plugin_requests {
 #include "pretag.h"
 #include "cfg.h"
 #include "util.h"
+#include "xflow_status.h"
 #include "log.h"
 #include "once.h"
 #include "mpls.h"
@@ -213,6 +214,12 @@ struct largebuf {
   u_char *ptr;
 };
 
+struct child_ctl {
+  u_int16_t active;
+  u_int16_t retired;
+  u_int32_t flags;
+};
+
 #define INIT_BUF(x) \
 	memset(x.base, 0, sizeof(x.base)); \
 	x.end = x.base+sizeof(x.base); \
@@ -232,6 +239,7 @@ void reload_maps();
 #else
 #define EXT
 #endif
+EXT void null_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
 EXT void eth_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
 EXT void fddi_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
 EXT void tr_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
@@ -263,6 +271,7 @@ EXT void compute_once();
 #endif
 EXT struct host_addr mcast_groups[MAX_MCAST_GROUPS];
 EXT int reload_map;
+EXT struct child_ctl sql_writers;
 #undef EXT
 
 size_t strlcpy(char *, const char *, size_t);
