@@ -57,7 +57,7 @@ void usage_daemon(char *prog_name)
   printf("  -N  \tDisable promiscuous mode\n");
   printf("  -n  \tPath to a file containing Network definitions\n");
   printf("  -o  \tPath to a file containing Port definitions\n");
-  printf("  -P  \t[ memory | print | mysql | pgsql | sqlite3 ] \n\tActivate plugin\n"); 
+  printf("  -P  \t[ memory | print | mysql | pgsql | sqlite3 | nfprobe | sfprobe ] \n\tActivate plugin\n"); 
   printf("  -d  \tEnable debug\n");
   printf("  -i  \tListen on the specified interface\n");
   printf("  -I  \tRead packets from the specified savefile\n");
@@ -125,6 +125,7 @@ int main(int argc,char **argv, char **envp)
   config.acct_type = ACCT_PM;
 
   rows = 0;
+  glob_pcapt = NULL;
 
   /* getting commandline values */
   while (!errflag && ((cp = getopt(argc, argv, ARGS_PMACCTD)) != -1)) {
@@ -267,6 +268,7 @@ int main(int argc,char **argv, char **envp)
 
   /* Let's check whether we need superuser privileges */
   if (config.snaplen) psize = config.snaplen;
+  else config.snaplen = psize;
 
   if (!config.pcap_savefile) {
     if (getuid() != 0) {
@@ -298,43 +300,94 @@ int main(int argc,char **argv, char **envp)
     Log(LOG_INFO, "INFO ( default/core ): Start logging ...\n");
   }
 
-  if (config.classifiers_path && config.sampling_rate) {
-    Log(LOG_ERR, "ERROR: Packet sampling and classification are actually mutual exclusive.\n");
-    exit(1);
-  }
+  if (config.logfile) config.logfile_fd = open_logfile(config.logfile);
 
   /* Enforcing policies over aggregation methods */
   list = plugins_list;
   while (list) {
-    if (strcmp(list->type.string, "core")) {
-      evaluate_sums(&list->cfg.what_to_count, list->name, list->type.string);
-      if (list->cfg.what_to_count & (COUNT_SRC_PORT|COUNT_DST_PORT|COUNT_SUM_PORT))
+    if (list->type.id != PLUGIN_ID_CORE) {
+      if (list->type.id == PLUGIN_ID_NFPROBE) {
+	if (config.classifiers_path && list->cfg.sampling_rate) {
+	  Log(LOG_ERR, "ERROR: Packet sampling and classification are actually mutual exclusive.\n");
+	  exit(1);
+	}
 	config.handle_fragments = TRUE;
-      if (list->cfg.what_to_count & COUNT_FLOWS) {
-	config.handle_fragments = TRUE;
-	config.handle_flows = TRUE;
-      }
-      if (list->cfg.what_to_count & COUNT_CLASS) {
-        config.handle_fragments = TRUE;
-        config.handle_flows = TRUE;
-      }
-      if (!list->cfg.what_to_count) {
-	Log(LOG_WARNING, "WARN ( %s/%s ): defaulting to SRC HOST aggregation.\n", list->name, list->type.string);
+	list->cfg.nfprobe_what_to_count = list->cfg.what_to_count;
+	list->cfg.what_to_count = 0;
+#if defined (HAVE_L2)
+	if (list->cfg.nfprobe_version == 9) {
+	  list->cfg.what_to_count |= COUNT_SRC_MAC;
+	  list->cfg.what_to_count |= COUNT_DST_MAC;
+	  list->cfg.what_to_count |= COUNT_VLAN;
+	}
+#endif
 	list->cfg.what_to_count |= COUNT_SRC_HOST;
+	list->cfg.what_to_count |= COUNT_DST_HOST;
+	list->cfg.what_to_count |= COUNT_SRC_PORT;
+	list->cfg.what_to_count |= COUNT_DST_PORT;
+	list->cfg.what_to_count |= COUNT_IP_TOS;
+	list->cfg.what_to_count |= COUNT_IP_PROTO;
+	if (list->cfg.networks_file) {
+	  list->cfg.what_to_count |= COUNT_SRC_AS;
+	  list->cfg.what_to_count |= COUNT_DST_AS;
+	}
+	if (list->cfg.nfprobe_version == 9 && list->cfg.classifiers_path) {
+	  list->cfg.what_to_count |= COUNT_CLASS; 
+	  config.handle_flows = TRUE;
+	}
+	list->cfg.what_to_count |= COUNT_COUNTERS;
+
+	list->cfg.data_type = PIPE_TYPE_METADATA;
+	list->cfg.data_type |= PIPE_TYPE_EXTRAS;
       }
-      if ((list->cfg.what_to_count & (COUNT_SRC_AS|COUNT_DST_AS|COUNT_SUM_AS)) && !list->cfg.networks_file) { 
-        Log(LOG_ERR, "ERROR ( %s/%s ): AS aggregation has been selected but NO 'networks_file' has been specified. Exiting...\n\n", list->name, list->type.string);
-        exit(1);
+      else if (list->type.id == PLUGIN_ID_SFPROBE) {
+        config.handle_fragments = FALSE;
+	if (psize < 128) psize = config.snaplen = 128; /* SFL_DEFAULT_HEADER_SIZE */
+	list->cfg.what_to_count = COUNT_PAYLOAD;
+	if (list->cfg.classifiers_path) {
+	  list->cfg.what_to_count |= COUNT_CLASS;
+	  config.handle_fragments = TRUE;
+	  config.handle_flows = TRUE;
+	}
+
+	list->cfg.data_type = PIPE_TYPE_PAYLOAD;
       }
-      if ((list->cfg.what_to_count & (COUNT_SRC_NET|COUNT_DST_NET|COUNT_SUM_NET)) && !list->cfg.networks_file && !list->cfg.networks_mask) {
-	Log(LOG_ERR, "ERROR ( %s/%s ): NET aggregation has been selected but NO 'networks_file' has been specified. Exiting...\n\n", list->name, list->type.string);
-	exit(1);
+      else {
+        if (config.classifiers_path && list->cfg.sampling_rate) {
+	  Log(LOG_ERR, "ERROR: Packet sampling and classification are actually mutual exclusive.\n");
+	  exit(1);
+        }
+	evaluate_sums(&list->cfg.what_to_count, list->name, list->type.string);
+	if (list->cfg.what_to_count & (COUNT_SRC_PORT|COUNT_DST_PORT|COUNT_SUM_PORT))
+	  config.handle_fragments = TRUE;
+	if (list->cfg.what_to_count & COUNT_FLOWS) {
+	  config.handle_fragments = TRUE;
+	  config.handle_flows = TRUE;
+	}
+	if (list->cfg.what_to_count & COUNT_CLASS) {
+	  config.handle_fragments = TRUE;
+	  config.handle_flows = TRUE;
+	}
+	if (!list->cfg.what_to_count) {
+	  Log(LOG_WARNING, "WARN ( %s/%s ): defaulting to SRC HOST aggregation.\n", list->name, list->type.string);
+	  list->cfg.what_to_count |= COUNT_SRC_HOST;
+	}
+	if ((list->cfg.what_to_count & (COUNT_SRC_AS|COUNT_DST_AS|COUNT_SUM_AS)) && !list->cfg.networks_file) { 
+	  Log(LOG_ERR, "ERROR ( %s/%s ): AS aggregation has been selected but NO 'networks_file' has been specified. Exiting...\n\n", list->name, list->type.string);
+	  exit(1);
+	}
+	if ((list->cfg.what_to_count & (COUNT_SRC_NET|COUNT_DST_NET|COUNT_SUM_NET)) && !list->cfg.networks_file && !list->cfg.networks_mask) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): NET aggregation has been selected but NO 'networks_file' has been specified. Exiting...\n\n", list->name, list->type.string);
+	  exit(1);
+	}
+	if (list->cfg.what_to_count & COUNT_CLASS && !list->cfg.classifiers_path) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): 'class' aggregation selected but NO 'classifiers' key specified. Exiting...\n\n", list->name, list->type.string);
+	  exit(1);
+	}
+	list->cfg.what_to_count |= COUNT_COUNTERS;
+	list->cfg.data_type = PIPE_TYPE_METADATA;
       }
-      if (list->cfg.what_to_count & COUNT_CLASS && !list->cfg.classifiers_path) {
-        Log(LOG_ERR, "ERROR ( %s/%s ): 'class' aggregation selected but NO 'classifiers' key specified. Exiting...\n\n", list->name, list->type.string);
-        exit(1);
-      }
-    } 
+    }
     list = list->next;
   }
 
@@ -406,6 +459,7 @@ int main(int argc,char **argv, char **envp)
     if (device.link_type == _devices[index].link_type)
       device.data = &_devices[index];
   }
+  load_plugin_filters(device.link_type);
 
   /* we need to solve some link constraints */
   if (device.data == NULL) {
@@ -513,7 +567,8 @@ void pcap_cb(u_char *user, const struct pcap_pkthdr *pkthdr, const u_char *buf)
     pptrs.pkthdr = (struct pcap_pkthdr *) pkthdr;
     pptrs.packet_ptr = (u_char *) buf;
     pptrs.mac_ptr = 0; pptrs.vlan_ptr = 0; pptrs.mpls_ptr = 0;
-    pptrs.pf = 0;
+    pptrs.pf = 0; pptrs.shadow = 0; pptrs.tag_dist = 1; pptrs.tag = 0;
+    pptrs.class = 0;
     pptrs.idtable = cb_data->idt;
     (*device->data->handler)(pkthdr, &pptrs);
     if (pptrs.iph_ptr) {
@@ -600,6 +655,11 @@ int ip_handler(register struct packet_ptrs *pptrs)
 
       ip_flow_handler(pptrs);
     }
+
+    /* XXX: optimize/short circuit here! */
+    pptrs->tcp_flags = FALSE;
+    if (pptrs->l4_proto == IPPROTO_TCP && off_l4+TCPFlagOff+1 <= caplen)
+      pptrs->tcp_flags = ((struct my_tcphdr *)pptrs->tlh_ptr)->th_flags; 
   }
 
   quit:
@@ -710,6 +770,11 @@ int ip6_handler(register struct packet_ptrs *pptrs)
 
       ip_flow6_handler(pptrs);
     }
+
+    /* XXX: optimize/short circuit here! */
+    pptrs->tcp_flags = FALSE;
+    if (pptrs->l4_proto == IPPROTO_TCP && off_l4+TCPFlagOff+1 <= caplen)
+      pptrs->tcp_flags = ((struct my_tcphdr *)pptrs->tlh_ptr)->th_flags;
   }
 
   quit:
@@ -725,7 +790,21 @@ int PM_find_id(struct packet_ptrs *pptrs)
   id = 0;
   for (x = 0; x < t->ipv4_num; x++) {
     for (j = 0, stop = 0; !stop; j++) stop = (*t->e[x].func[j])(pptrs, &id, &t->e[x]);
-    if (id) break;
+    if (id) {
+      if (t->e[x].stack.func) id = (*t->e[x].stack.func)(id, pptrs->tag);
+      pptrs->tag = id;
+      pptrs->tag_dist = t->e[x].ret; 
+
+      if (t->e[x].jeq.ptr) {
+	exec_plugins(pptrs);
+
+	x = t->e[x].jeq.ptr->pos;
+	x--; /* yes, it will be automagically incremented by the for() cycle */
+	if (t->e[x].ret) set_shadow_status(pptrs);
+	id = 0;
+      }
+      else break;
+    }
   }
 
   return id;
@@ -737,6 +816,8 @@ void compute_once()
 
   CounterSz = sizeof(dummy.pkt_len);
   PdataSz = sizeof(struct pkt_data);
+  PpayloadSz = sizeof(struct pkt_payload);
+  PextrasSz = sizeof(struct pkt_extras);
   ChBufHdrSz = sizeof(struct ch_buf_hdr);
   CharPtrSz = sizeof(char *);
   IP4HdrSz = sizeof(struct my_iphdr);
