@@ -28,15 +28,15 @@
 #include "mysql_plugin.h"
 #include "ip_flow.h"
 #include "classifier.h"
-#include "util.h"
 
-#define ARGS "df:o:n:thiP:T:U:D:H:"
+#define ARGS "df:o:n:thieP:T:U:D:H:"
 
 struct DBdesc db;
 struct logfile_header lh;
 int re = 0, we = 0;
 int debug = 0;
 int sql_dont_try_update = 0;
+int sql_history_since_epoch = 0;
 char timebuf[SRVBUFLEN];
 char *sql_table;
 
@@ -56,6 +56,7 @@ void usage(char *prog)
   printf("  -D\t[ DB ]\n\tUse the specified SQL database\n");
   printf("  -T\t[ table ]\n\tUse the specified SQL table\n");
   printf("  -i\tDon't try update, use insert only.\n");
+  printf("  -e\tUse seconds since the Epoch timestamps.\n");
   printf("\n");
   printf("For suggestions, critics, bugs, contact me: %s.\n", MANTAINER);
 }
@@ -136,9 +137,12 @@ void print_data(struct db_cache *cache_elem, u_int32_t wtc, int num)
   printf("%-10lu  ", cache_elem->bytes_counter);
 #endif
   if (lh.sql_history) {
-    lt = localtime(&cache_elem->basetime); 
-    strftime(timebuf, SRVBUFLEN, "%Y-%m-%d %H:%M:%S" , lt); 
-    printf("%s\n", timebuf);
+    if (!sql_history_since_epoch) {
+      lt = localtime(&cache_elem->basetime); 
+      strftime(timebuf, SRVBUFLEN, "%Y-%m-%d %H:%M:%S" , lt); 
+      printf("%s\n", timebuf);
+    }
+    else printf("%u\n", cache_elem->basetime);
   }
   else printf("0\n"); 
 }
@@ -215,6 +219,9 @@ int main(int argc, char **argv)
     case 'i':
       sql_dont_try_update = TRUE;
       break;
+    case 'e':
+      sql_history_since_epoch = TRUE;
+      break;
     case 'P':
       strlcpy(sql_pwd, optarg, sizeof(sql_pwd));
       have_pwd = TRUE;
@@ -442,11 +449,18 @@ int MY_evaluate_history(int primitive)
       strncat(values[primitive].string, ", ", sizeof(values[primitive].string));
       strncat(where[primitive].string, " AND ", sizeof(where[primitive].string));
     }
-    strncat(where[primitive].string, "FROM_UNIXTIME(%u) = ", SPACELEFT(where[primitive].string));
+    if (!sql_history_since_epoch)
+      strncat(where[primitive].string, "FROM_UNIXTIME(%u) = ", SPACELEFT(where[primitive].string));
+    else
+      strncat(where[primitive].string, "%u = ", SPACELEFT(where[primitive].string));
+	 
     strncat(where[primitive].string, "stamp_inserted", SPACELEFT(where[primitive].string));
 
     strncat(insert_clause, "stamp_updated, stamp_inserted", SPACELEFT(insert_clause));
-    strncat(values[primitive].string, "FROM_UNIXTIME(%u), FROM_UNIXTIME(%u)", SPACELEFT(values[primitive].string));
+    if (!sql_history_since_epoch)
+      strncat(values[primitive].string, "FROM_UNIXTIME(%u), FROM_UNIXTIME(%u)", SPACELEFT(values[primitive].string));
+    else
+      strncat(values[primitive].string, "%u, %u", SPACELEFT(values[primitive].string));
 
     where[primitive].type = values[primitive].type = TIMESTAMP;
     values[primitive].handler = where[primitive].handler = count_timestamp_handler;
@@ -757,7 +771,12 @@ int MY_compose_static_queries()
   strncat(update_clause, "SET packets=packets+%lu, bytes=bytes+%lu", SPACELEFT(update_clause));
   if (have_flows) strncat(update_clause, ", flows=flows+%lu", SPACELEFT(update_clause));
 #endif
-  if (lh.sql_history) strncat(update_clause, ", stamp_updated=now()", SPACELEFT(update_clause));
+  if (lh.sql_history) {
+    if (!sql_history_since_epoch)
+      strncat(update_clause, ", stamp_updated=NOW()", SPACELEFT(update_clause));
+    else
+      strncat(update_clause, ", stamp_updated=UNIX_TIMESTAMP(NOW())", SPACELEFT(update_clause));
+  }
 
   return primitives;
 }
