@@ -64,7 +64,7 @@ void usage_daemon(char *prog_name)
   printf("  -D  \tDaemonize\n"); 
   printf("  -n  \tPath to a file containing Network definitions\n");
   printf("  -o  \tPath to a file containing Port definitions\n");
-  printf("  -P  \t[ memory | print | mysql | pgsql | sqlite3 ] \n\tActivate plugin\n"); 
+  printf("  -P  \t[ memory | print | mysql | pgsql | sqlite3 | nfprobe | sfprobe ] \n\tActivate plugin\n"); 
   printf("  -d  \tEnable debug\n");
   printf("  -S  \t[ auth | mail | daemon | kern | user | local[0-7] ] \n\ttLog to the specified syslog facility\n");
   printf("  -F  \tWrite Core Process PID into the specified file\n");
@@ -145,7 +145,8 @@ int main(int argc,char **argv, char **envp)
   /* a bunch of default definitions */ 
   have_num_memory_pools = FALSE;
   reload_map = FALSE;
-  renorm_table_entries = 0;
+  xflow_status_table_entries = 0;
+  xflow_tot_bad_datagrams = 0;
   errflag = 0;
 
   memset(cfg_cmdline, 0, sizeof(cfg_cmdline));
@@ -156,11 +157,12 @@ int main(int argc,char **argv, char **envp)
   memset(&pptrs, 0, sizeof(pptrs));
   memset(&req, 0, sizeof(req));
   memset(&spp, 0, sizeof(spp));
-  memset(&renorm_table, 0, sizeof(renorm_table));
   memset(&class, 0, sizeof(class));
+  memset(&xflow_status_table, 0, sizeof(xflow_status_table));
   config.acct_type = ACCT_SF;
 
   rows = 0;
+  glob_pcapt = NULL;
 
   /* getting commandline values */
   while (!errflag && ((cp = getopt(argc, argv, ARGS_SFACCTD)) != -1)) {
@@ -310,31 +312,74 @@ int main(int argc,char **argv, char **envp)
     Log(LOG_INFO, "INFO ( default/core ): Start logging ...\n");
   }
 
+  if (config.logfile) config.logfile_fd = open_logfile(config.logfile);
+
   /* Enforcing policies over aggregation methods */
   list = plugins_list;
   while (list) {
-    if (strcmp(list->type.string, "core")) {  
-      evaluate_sums(&list->cfg.what_to_count, list->name, list->type.string);
-      if (!list->cfg.what_to_count) {
-	Log(LOG_WARNING, "WARN ( %s/%s ): defaulting to SRC HOST aggregation.\n", list->name, list->type.string);
+    if (list->type.id != PLUGIN_ID_CORE) {  
+      if (list->type.id == PLUGIN_ID_NFPROBE) {
+	list->cfg.nfprobe_what_to_count = list->cfg.what_to_count;
+	list->cfg.what_to_count = 0;
+#if defined (HAVE_L2)
+	if (list->cfg.nfprobe_version == 9) {
+	  list->cfg.what_to_count |= COUNT_SRC_MAC;
+	  list->cfg.what_to_count |= COUNT_DST_MAC;
+	  list->cfg.what_to_count |= COUNT_VLAN;
+	}
+#endif
 	list->cfg.what_to_count |= COUNT_SRC_HOST;
+	list->cfg.what_to_count |= COUNT_DST_HOST;
+	list->cfg.what_to_count |= COUNT_SRC_PORT;
+	list->cfg.what_to_count |= COUNT_DST_PORT;
+	list->cfg.what_to_count |= COUNT_IP_TOS;
+	list->cfg.what_to_count |= COUNT_IP_PROTO;
+	if (list->cfg.networks_file || list->cfg.nfacctd_as == NF_AS_KEEP) {
+	  list->cfg.what_to_count |= COUNT_SRC_AS;
+	  list->cfg.what_to_count |= COUNT_DST_AS;
+	}
+	if (list->cfg.nfprobe_version == 9 && list->cfg.classifiers_path)
+	  list->cfg.what_to_count |= COUNT_CLASS;
+	list->cfg.what_to_count |= COUNT_COUNTERS;
+
+	list->cfg.data_type = PIPE_TYPE_METADATA;
+	list->cfg.data_type |= PIPE_TYPE_EXTRAS;
       }
-      if ((list->cfg.what_to_count & (COUNT_SRC_AS|COUNT_DST_AS|COUNT_SUM_AS)) && !list->cfg.networks_file && list->cfg.nfacctd_as != NF_AS_KEEP) {
-        Log(LOG_ERR, "ERROR ( %s/%s ): AS aggregation has been selected but NO 'networks_file' has been specified. Exiting...\n\n", list->name, list->type.string);
-        exit(1);
+      else if (list->type.id == PLUGIN_ID_SFPROBE) {
+	list->cfg.what_to_count = COUNT_PAYLOAD;
+	if (list->cfg.classifiers_path) list->cfg.what_to_count |= COUNT_CLASS;
+
+	list->cfg.data_type = PIPE_TYPE_PAYLOAD;
       }
-      if ((list->cfg.what_to_count & (COUNT_SRC_NET|COUNT_DST_NET|COUNT_SUM_NET)) && !list->cfg.networks_file && !list->cfg.networks_mask) {
-        Log(LOG_ERR, "ERROR ( %s/%s ): NET aggregation has been selected but NO 'networks_file' has been specified. Exiting...\n\n", list->name, list->type.string);
-        exit(1);
+      else {
+	evaluate_sums(&list->cfg.what_to_count, list->name, list->type.string);
+	if (!list->cfg.what_to_count) {
+	  Log(LOG_WARNING, "WARN ( %s/%s ): defaulting to SRC HOST aggregation.\n", list->name, list->type.string);
+	  list->cfg.what_to_count |= COUNT_SRC_HOST;
+	}
+	if ((list->cfg.what_to_count & (COUNT_SRC_AS|COUNT_DST_AS|COUNT_SUM_AS)) && !list->cfg.networks_file && list->cfg.nfacctd_as != NF_AS_KEEP) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): AS aggregation has been selected but NO 'networks_file' has been specified. Exiting...\n\n", list->name, list->type.string);
+	  exit(1);
+	}
+	if ((list->cfg.what_to_count & (COUNT_SRC_NET|COUNT_DST_NET|COUNT_SUM_NET)) && !list->cfg.networks_file && !list->cfg.networks_mask) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): NET aggregation has been selected but NO 'networks_file' has been specified. Exiting...\n\n", list->name, list->type.string);
+	  exit(1);
+	}
+	if (list->cfg.what_to_count & COUNT_CLASS && !list->cfg.classifiers_path) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): 'class' aggregation selected but NO 'classifiers' key specified. Exiting...\n\n", list->name, list->type.string);
+	  exit(1);
+	}
+	list->cfg.what_to_count |= COUNT_COUNTERS;
+	list->cfg.data_type = PIPE_TYPE_METADATA;
       }
-    } 
+    }
     list = list->next;
   }
 
   /* signal handling we want to inherit to plugins (when not re-defined elsewhere) */
   signal(SIGCHLD, startup_handle_falling_child); /* takes note of plugins failed during startup phase */
   signal(SIGHUP, reload); /* handles reopening of syslog channel */
-  signal(SIGUSR1, SIG_IGN); /* ignore this signal */ 
+  signal(SIGUSR1, push_stats); /* logs various statistics via Log() calls */ 
   signal(SIGUSR2, reload_maps); /* sets to true the reload_maps flag */
   signal(SIGPIPE, SIG_IGN); /* we want to exit gracefully when a pipe is broken */
 
@@ -425,8 +470,11 @@ int main(int argc,char **argv, char **envp)
     pptrs.v4.idtable = NULL;
   }
 
+  if (config.classifiers_path) init_classifiers(config.classifiers_path);
+
   /* plugins glue: creation */
   load_plugins(&req);
+  load_plugin_filters(1);
   evaluate_packet_handlers();
   pm_setproctitle("%s [%s]", "Core Process", "default");
   if (config.pidfile) write_pid_file(config.pidfile);
@@ -576,8 +624,11 @@ int main(int argc,char **argv, char **envp)
     spp.rawSampleLen = ret;
     spp.datap = (u_int32_t *) spp.rawSample;
     spp.endp = sflow_packet + spp.rawSampleLen; 
+    reset_tag_status(&pptrs);
+    reset_shadow_status(&pptrs);
+    reset_tagdist_status(&pptrs);
 
-    if (ret < SFLOW_MIN_MSG_SIZE) continue; 
+    // if (ret < SFLOW_MIN_MSG_SIZE) continue; 
 
     /* check if Hosts Allow Table is loaded; if it is, we will enforce rules */
     if (allow.num) allowed = check_allow(&allow, (struct sockaddr *)&client); 
@@ -592,15 +643,16 @@ int main(int argc,char **argv, char **envp)
     switch(spp.datagramVersion = getData32(&spp)) {
     case 5:
       getAddress(&spp, &spp.agent_addr);
-      process_SFv5_packet(&spp, &pptrs, &req);
+      process_SFv5_packet(&spp, &pptrs, &req, (struct sockaddr *) &client);
       break;
     case 4:
     case 2:
       getAddress(&spp, &spp.agent_addr);
-      process_SFv2v4_packet(&spp, &pptrs, &req);
+      process_SFv2v4_packet(&spp, &pptrs, &req, (struct sockaddr *) &client);
       break;
     default:
       notify_malf_packet(LOG_INFO, "INFO: Discarding unknown packet", (struct sockaddr *) pptrs.v4.f_agent);
+      xflow_tot_bad_datagrams++;
       break;
     }
   }
@@ -615,14 +667,18 @@ void InterSampleCleanup(SFSample *spp)
 }
 
 void process_SFv2v4_packet(SFSample *spp, struct packet_ptrs_vector *pptrsv,
-		                struct plugin_requests *req)
+		                struct plugin_requests *req, struct sockaddr *agent)
 {
   u_int32_t samplesInPacket, idx;
   u_int32_t sampleType;
 
+  spp->agentSubId = 0; /* not supported */
   spp->sequenceNo = getData32(spp);
   spp->sysUpTime = getData32(spp);
   samplesInPacket = getData32(spp);
+  
+  pptrsv->v4.f_status = sfv245_check_status(spp, agent);
+  set_vector_f_status(pptrsv);
 
   for (idx = 0; idx < samplesInPacket; idx++) {
     InterSampleCleanup(spp);
@@ -636,6 +692,7 @@ void process_SFv2v4_packet(SFSample *spp, struct packet_ptrs_vector *pptrsv,
       break;
     default:
       notify_malf_packet(LOG_INFO, "INFO: Discarding unknown v2/v4 sample", (struct sockaddr *) pptrsv->v4.f_agent);
+      xflow_tot_bad_datagrams++;
       return; /* unexpected sampleType; aborting packet */
     }
     if ((u_char *)spp->datap > spp->endp) return;
@@ -643,7 +700,7 @@ void process_SFv2v4_packet(SFSample *spp, struct packet_ptrs_vector *pptrsv,
 }
 
 void process_SFv5_packet(SFSample *spp, struct packet_ptrs_vector *pptrsv,
-		struct plugin_requests *req)
+		struct plugin_requests *req, struct sockaddr *agent)
 {
   u_int32_t samplesInPacket, idx;
   u_int32_t sampleType;
@@ -652,6 +709,8 @@ void process_SFv5_packet(SFSample *spp, struct packet_ptrs_vector *pptrsv,
   spp->sequenceNo = getData32(spp);
   spp->sysUpTime = getData32(spp);
   samplesInPacket = getData32(spp);
+  pptrsv->v4.f_status = sfv245_check_status(spp, agent);
+  set_vector_f_status(pptrsv);
 
   for (idx = 0; idx < samplesInPacket; idx++) {
     InterSampleCleanup(spp);
@@ -671,6 +730,7 @@ void process_SFv5_packet(SFSample *spp, struct packet_ptrs_vector *pptrsv,
       break;
     default:
       notify_malf_packet(LOG_INFO, "INFO: Discarding unknown v5 sample", (struct sockaddr *) pptrsv->v4.f_agent);
+      xflow_tot_bad_datagrams++;
       return; /* unexpected sampleType; aborting packet */ 
     }
     if ((u_char *)spp->datap > spp->endp) return; 
@@ -732,13 +792,15 @@ void compute_once()
 
   CounterSz = sizeof(dummy.pkt_len);
   PdataSz = sizeof(struct pkt_data);
+  PpayloadSz = sizeof(struct pkt_payload);
+  PextrasSz = sizeof(struct pkt_extras);
   ChBufHdrSz = sizeof(struct ch_buf_hdr);
   CharPtrSz = sizeof(char *);
   IP4HdrSz = sizeof(struct my_iphdr);
   IP4TlSz = sizeof(struct my_iphdr)+sizeof(struct my_tlhdr);
   SFSampleSz = sizeof(SFSample);
   SFLAddressSz = sizeof(SFLAddress);
-  SFrenormEntrySz = sizeof(struct SF_renorm_entry);
+  SFrenormEntrySz = sizeof(struct xflow_status_entry_sampling);
   PptrsSz = sizeof(struct packet_ptrs);
   CSSz = sizeof(struct class_st);
   HostAddrSz = sizeof(struct host_addr);
@@ -1382,6 +1444,31 @@ void readExtendedProcess(SFSample *sample)
   for (i = 0; i < num_processes; i++) skipBytes(sample, 4);
 }
 
+void readExtendedClass(SFSample *sample)
+{
+  u_int32_t ret;
+  u_char buf[MAX_PROTOCOL_LEN+1], *bufptr = buf;
+
+  if (config.classifiers_path) {
+    ret = getData32_nobswap(sample);
+    memcpy(bufptr, &ret, 4);
+    bufptr += 4;
+    ret = getData32_nobswap(sample);
+    memcpy(bufptr, &ret, 4);
+    bufptr += 4;
+    ret = getData32_nobswap(sample);
+    memcpy(bufptr, &ret, 4);
+    bufptr += 4;
+    ret = getData32_nobswap(sample);
+    memcpy(bufptr, &ret, 4);
+    bufptr += 4;
+
+    sample->class = NF_evaluate_classifiers(buf);
+  }
+  else skipBytes(sample, MAX_PROTOCOL_LEN);
+}
+
+
 void decodeMpls(SFSample *sample)
 {
   struct packet_ptrs dummy_pptrs;
@@ -1600,7 +1687,10 @@ void readv2v4FlowSample(SFSample *sample, struct packet_ptrs_vector *pptrsv, str
   case INMPACKETTYPE_HEADER: readFlowSample_header(sample); break;
   case INMPACKETTYPE_IPV4: readFlowSample_IPv4(sample); break;
   case INMPACKETTYPE_IPV6: readFlowSample_IPv6(sample); break;
-  default: notify_malf_packet(LOG_INFO, "INFO: Discarding unknown v2/v4 Data Tag", (struct sockaddr *) pptrsv->v4.f_agent); break;
+  default: 
+    notify_malf_packet(LOG_INFO, "INFO: Discarding unknown v2/v4 Data Tag", (struct sockaddr *) pptrsv->v4.f_agent);
+    xflow_tot_bad_datagrams++;
+    break;
   }
 
   sample->extended_data_tag = 0;
@@ -1619,7 +1709,10 @@ void readv2v4FlowSample(SFSample *sample, struct packet_ptrs_vector *pptrsv, str
 	break;
       case INMEXTENDED_USER: readExtendedUser(sample); break;
       case INMEXTENDED_URL: readExtendedUrl(sample); break;
-      default: notify_malf_packet(LOG_INFO, "INFO: Discarding unknown v2/v4 Extended Data Tag", (struct sockaddr *) pptrsv->v4.f_agent); break;
+      default: 
+	notify_malf_packet(LOG_INFO, "INFO: Discarding unknown v2/v4 Extended Data Tag", (struct sockaddr *) pptrsv->v4.f_agent);
+	xflow_tot_bad_datagrams++;
+	break;
       }
     }
   }
@@ -1697,6 +1790,7 @@ void readv5FlowSample(SFSample *sample, int expanded, struct packet_ptrs_vector
       case SFLFLOW_EX_MPLS_LDP_FEC: readExtendedMplsLDP_FEC(sample); break;
       case SFLFLOW_EX_VLAN_TUNNEL:  readExtendedVlanTunnel(sample); break;
       case SFLFLOW_EX_PROCESS:      readExtendedProcess(sample); break;
+      case SFLFLOW_EX_CLASS:	    readExtendedClass(sample); break;
       default: skipBytes(sample, length); break; 
       }
       if (lengthCheck(sample, start, length) == ERR) return;
@@ -1864,7 +1958,7 @@ void finalizeSample(SFSample *sample, struct packet_ptrs_vector *pptrsv, struct
         memcpy(pptrsv->mpls4.mac_ptr, &sample->eth_dst, ETH_ADDR_LEN); 
 
         for (idx = 0; idx <= sample->lstk.depth && idx < 10; idx++) { 
-          label = htonl(sample->lstk.stack[idx]);
+          label = sample->lstk.stack[idx];
           memcpy(ptr, &label, 4);
           ptr += 4;
         }
@@ -1898,7 +1992,7 @@ void finalizeSample(SFSample *sample, struct packet_ptrs_vector *pptrsv, struct
         memcpy(pptrsv->mpls6.mac_ptr, &sample->eth_dst, ETH_ADDR_LEN); 
 
 	for (idx = 0; idx <= sample->lstk.depth && idx < 10; idx++) {
-	  label = htonl(sample->lstk.stack[idx]);
+	  label = sample->lstk.stack[idx];
 	  memcpy(ptr, &label, 4);
 	  ptr += 4;
 	}
@@ -1933,7 +2027,7 @@ void finalizeSample(SFSample *sample, struct packet_ptrs_vector *pptrsv, struct
         memcpy(pptrsv->vlanmpls4.vlan_ptr, &in_vlan, 2); 
 
 	for (idx = 0; idx <= sample->lstk.depth && idx < 10; idx++) {
-	  label = htonl(sample->lstk.stack[idx]);
+	  label = sample->lstk.stack[idx];
 	  memcpy(ptr, &label, 4);
 	  ptr += 4;
 	}
@@ -1968,7 +2062,7 @@ void finalizeSample(SFSample *sample, struct packet_ptrs_vector *pptrsv, struct
         memcpy(pptrsv->vlanmpls6.vlan_ptr, &in_vlan, 2); 
 
 	for (idx = 0; idx <= sample->lstk.depth && idx < 10; idx++) {
-	  label = htonl(sample->lstk.stack[idx]);
+	  label = sample->lstk.stack[idx];
 	  memcpy(ptr, &label, 4);
 	  ptr += 4;
 	}
@@ -2011,22 +2105,46 @@ int SF_find_id(struct packet_ptrs *pptrs)
   id = 0;
   if (sample->agent_addr.type == SFLADDRESSTYPE_IP_V4) {
     for (x = 0; x < t->ipv4_num; x++) {
-      if (t->e[x].agent_ip.address.ipv4.s_addr == sample->agent_addr.address.ip_v4.s_addr) {
+      if (t->e[x].agent_ip.a.address.ipv4.s_addr == sample->agent_addr.address.ip_v4.s_addr) {
         for (j = 0, stop = 0; !stop; j++) stop = (*t->e[x].func[j])(pptrs, &id, &t->e[x]);
-        if (id) break;
+        if (id) {
+	  if (t->e[x].stack.func) id = (*t->e[x].stack.func)(id, pptrs->tag);
+          pptrs->tag = id;
+	  pptrs->tag_dist = t->e[x].ret;
+          if (t->e[x].jeq.ptr) {
+            exec_plugins(pptrs);
+
+            x = t->e[x].jeq.ptr->pos;
+            x--; /* yes, it will be automagically incremented by the for() cycle */
+            if (t->e[x].ret) set_shadow_status(pptrs);
+            id = 0;
+          }
+          else break;
+        }
       }
-      else if (t->e[x].agent_ip.address.ipv4.s_addr > sample->agent_addr.address.ip_v4.s_addr) break;
     }
   }
 #if defined ENABLE_IPV6
   else if (sample->agent_addr.type == SFLADDRESSTYPE_IP_V6) { 
     for (x = (t->num-t->ipv6_num); x < t->num; x++) {
-      if (!ip6_addr_cmp(&t->e[x].agent_ip.address.ipv6, &sample->agent_addr.address.ip_v6)) {
+      if (!ip6_addr_cmp(&t->e[x].agent_ip.a.address.ipv6, &sample->agent_addr.address.ip_v6)) {
         for (j = 0, stop = 0; !stop; j++) stop = (*t->e[x].func[j])(pptrs, &id, &t->e[x]);
-        if (id) break;
+        if (id) {
+	  if (t->e[x].stack.func) id = (*t->e[x].stack.func)(id, pptrs->tag);
+          pptrs->tag = id;
+	  pptrs->tag_dist = t->e[x].ret;
+
+          if (t->e[x].jeq.ptr) {
+            exec_plugins(pptrs);
+
+            x = t->e[x].jeq.ptr->pos;
+            x--; /* yes, it will be automagically incremented by the for() cycle */
+            if (t->e[x].ret) set_shadow_status(pptrs);
+            id = 0;
+          }
+          else break;
+        }
       }
-      else if (ip6_addr_cmp(&t->e[x].agent_ip.address.ipv6, &sample->agent_addr.address.ip_v6) > 0)
-        break;
     }
   }
 #endif
@@ -2081,3 +2199,17 @@ int ip6_handler(register struct packet_ptrs *pptrs)
 {
 }
 
+char *sfv245_check_status(SFSample *spp, struct sockaddr *sa)
+{
+  u_int32_t aux1 = spp->agentSubId;
+  int hash = hash_status_table(aux1, sa, XFLOW_STATUS_TABLE_SZ);
+  struct xflow_status_entry *entry = NULL;
+
+  if (hash >= 0) {
+    entry = search_status_table(sa, aux1, hash, XFLOW_STATUS_TABLE_MAX_ENTRIES);
+    update_status_table(entry, spp->sequenceNo);
+    entry->inc = 1;
+  }
+
+  return (char *) entry;
+}
