@@ -26,7 +26,6 @@
 #include "pmacct-data.h"
 #include "plugin_hooks.h"
 #include "sql_common.h"
-#include "util.h"
 #include "crc32.c"
 #include "sql_common_m.c"
 
@@ -38,11 +37,11 @@ void sql_set_signals()
   signal(SIGUSR1, SIG_IGN);
   signal(SIGUSR2, reload_maps);
   signal(SIGPIPE, SIG_IGN);
-#if !defined FBSD4
-  signal(SIGCHLD, SIG_IGN);
-#else
+//#if !defined FBSD4
+//  signal(SIGCHLD, SIG_IGN);
+//#else
   signal(SIGCHLD, ignore_falling_child);
-#endif
+//#endif
 }
 
 void sql_set_insert_func()
@@ -75,10 +74,8 @@ void sql_init_global_buffers()
   memset(lock_clause, 0, sizeof(lock_clause));
   memset(unlock_clause, 0, sizeof(unlock_clause));
   memset(update_clause, 0, sizeof(update_clause));
-  memset(update_negative_clause, 0, sizeof(update_negative_clause));
   memset(set_clause, 0, sizeof(set_clause));
-  memset(set_negative_clause, 0, sizeof(set_negative_clause));
-  memset(delete_shadows_clause, 0, sizeof(delete_shadows_clause));
+  memset(copy_clause, 0, sizeof(copy_clause));
   memset(insert_clause, 0, sizeof(insert_clause));
   memset(where, 0, sizeof(where));
   memset(values, 0, sizeof(values));
@@ -88,10 +85,12 @@ void sql_init_global_buffers()
   pipebuf = (unsigned char *) malloc(config.buffer_size);
   cache = (struct db_cache *) malloc(config.sql_cache_entries*sizeof(struct db_cache));
   queries_queue = (struct db_cache **) malloc(qq_size*sizeof(struct db_cache *));
+  pending_queries_queue = (struct db_cache **) malloc(qq_size*sizeof(struct db_cache *));
 
   memset(pipebuf, 0, config.buffer_size);
   memset(cache, 0, config.sql_cache_entries*sizeof(struct db_cache));
   memset(queries_queue, 0, qq_size*sizeof(struct db_cache *));
+  memset(pending_queries_queue, 0, qq_size*sizeof(struct db_cache *));
 }
 
 void sql_init_default_values()
@@ -107,11 +106,20 @@ void sql_init_default_values()
   if (!config.sql_refresh_time) config.sql_refresh_time = DEFAULT_DB_REFRESH_TIME;
   if (!config.sql_table_version) config.sql_table_version = DEFAULT_SQL_TABLE_VERSION;
   if (!config.sql_cache_entries) config.sql_cache_entries = CACHE_ENTRIES;
+  if (!config.sql_max_writers) config.sql_max_writers = DEFAULT_SQL_WRITERS_NO;
 
-  qq_ptr = 0;
+  if (config.sql_aggressive_classification) {
+    if (config.acct_type == ACCT_PM && config.what_to_count & COUNT_CLASS);
+    else config.sql_aggressive_classification = FALSE;
+  }
+
+  qq_ptr = 0; 
+  pqq_ptr = 0;
   qq_size = config.sql_cache_entries+(config.sql_refresh_time*REASONABLE_NUMBER);
   pp_size = sizeof(struct pkt_primitives);
   dbc_size = sizeof(struct db_cache);
+
+  memset(&sql_writers, 0, sizeof(sql_writers));
 }
 
 void sql_init_historical_acct(time_t now, struct insert_data *idata)
@@ -230,12 +238,48 @@ void sql_cache_modulo(struct pkt_primitives *srcdst, struct insert_data *idata)
   idata->modulo = idata->hash % config.sql_cache_entries;
 }
 
-int sql_cache_flush(struct db_cache *queue[], int index)
+int sql_cache_flush(struct db_cache *queue[], int index, struct insert_data *idata)
 {
-  int j;
+  int j, tmp_retired = sql_writers.retired;
 
-  for (j = 0; j < index; j++) queue[j]->valid = FALSE;
-  index = 0;
+  /* If aggressive classification is enabled and there are still
+     chances for the stream to be classified - ie. tentatives is
+     non-zero - let's leave it in SQL_CACHE_INUSE state */
+  if (config.sql_aggressive_classification) {
+    for (j = 0, pqq_ptr = 0; j < index; j++) {
+      if (!queue[j]->primitives.class && queue[j]->tentatives && (queue[j]->start_tag > (idata->now - ((STALE_M-1) * config.sql_refresh_time))) ) {
+        pending_queries_queue[pqq_ptr] = queue[j];
+        pqq_ptr++;
+      }
+      else queue[j]->valid = SQL_CACHE_COMMITTED;
+    }
+  }
+  else {
+    for (j = 0; j < index; j++) queue[j]->valid = SQL_CACHE_COMMITTED;
+  }
+
+  /* Imposing maximum number of writers */
+  sql_writers.active -= MIN(sql_writers.active, tmp_retired);
+  sql_writers.retired -= tmp_retired;
+
+  if (sql_writers.active < config.sql_max_writers) {
+    /* If we are very near to our maximum writers threshold, let's resort to any configured
+       recovery mechanism - SQL_CACHE_COMMITTED => SQL_CACHE_ERROR; otherwise, will proceed
+       as usual */
+    if (sql_writers.active == config.sql_max_writers-1) {
+      for (j = 0; j < index; j++) {
+	if (queue[j]->valid == SQL_CACHE_COMMITTED) queue[j]->valid = SQL_CACHE_ERROR;
+      }
+      sql_writers.flags = CHLD_WARNING;
+    }
+    else sql_writers.flags = 0; /* everything is just fine */
+
+    sql_writers.active++;
+  }
+  else {
+    Log(LOG_WARNING, "WARN ( %s/%s ): Maximum number of SQL writer processes reached (%d).\n", config.name, config.type, sql_writers.active);
+    sql_writers.flags = CHLD_ALERT;
+  }
 
   return index;
 }
@@ -253,7 +297,7 @@ struct db_cache *sql_cache_search(struct pkt_primitives *data, time_t basetime)
 
   start:
   if (idata.hash != Cursor->signature) {
-    if (Cursor->valid == TRUE) {
+    if (Cursor->valid == SQL_CACHE_INUSE) {
       follow_chain:
       if (Cursor->next) {
         Cursor = Cursor->next;
@@ -262,7 +306,7 @@ struct db_cache *sql_cache_search(struct pkt_primitives *data, time_t basetime)
     }
   }
   else {
-    if (Cursor->valid == TRUE) {
+    if (Cursor->valid == SQL_CACHE_INUSE) {
       /* additional check: pkt_primitives */
       if (!memcmp(&Cursor->primitives, data, sizeof(struct pkt_primitives))) {
         /* additional check: time */
@@ -304,7 +348,6 @@ void sql_cache_insert(struct pkt_data *data, struct insert_data *idata)
   /* We are classifing packets. We have a non-zero bytes accumulator (ba)
      and a non-zero class. Before accounting ba to this class, we have to
      remove ba from class zero. */
-  idata->pending_accumulators = FALSE;
   if (config.what_to_count & COUNT_CLASS && data->cst.ba && data->primitives.class) {
     pm_class_t lclass = data->primitives.class;
 
@@ -319,20 +362,13 @@ void sql_cache_insert(struct pkt_data *data, struct insert_data *idata)
 
     if (Cursor) {
       if (timeval_cmp(&data->cst.stamp, &idata->flushtime) >= 0) { 
-        Cursor->bytes_counter -= data->cst.ba;
-        Cursor->packet_counter -= data->cst.pa;
-        Cursor->flows_counter -= data->cst.fa;
-	idata->pending_accumulators = FALSE;
-      }
-      else {
-        if (!config.sql_aggressive_classification) memset(&data->cst, 0, CSSz);
-	else idata->pending_accumulators = TRUE;
+        Cursor->bytes_counter -= MIN(Cursor->bytes_counter, data->cst.ba);
+        Cursor->packet_counter -= MIN(Cursor->packet_counter, data->cst.pa);
+        Cursor->flows_counter -= MIN(Cursor->flows_counter, data->cst.fa);
       }
+      else memset(&data->cst, 0, CSSz);
     }
-    else {
-      if (!config.sql_aggressive_classification) memset(&data->cst, 0, CSSz); 
-      else idata->pending_accumulators = TRUE;
-    }
+    else memset(&data->cst, 0, CSSz); 
   }
 
   sql_cache_modulo(&data->primitives, idata);
@@ -346,7 +382,7 @@ void sql_cache_insert(struct pkt_data *data, struct insert_data *idata)
 
   start:
   if (idata->hash != Cursor->signature) {
-    if (Cursor->valid == TRUE) {
+    if (Cursor->valid == SQL_CACHE_INUSE) {
       follow_chain:
       if (Cursor->next) {
         Cursor = Cursor->next;
@@ -374,7 +410,7 @@ void sql_cache_insert(struct pkt_data *data, struct insert_data *idata)
     else goto insert; /* we found a no more valid entry; let's insert here our data */
   }
   else {
-    if (Cursor->valid == TRUE) {
+    if (Cursor->valid == SQL_CACHE_INUSE) {
       /* additional check: pkt_primitives */
       if (!memcmp(&Cursor->primitives, srcdst, sizeof(struct pkt_primitives))) {
         /* additional check: time */
@@ -407,22 +443,14 @@ void sql_cache_insert(struct pkt_data *data, struct insert_data *idata)
   Cursor->flows_counter = data->flo_num;
   Cursor->bytes_counter = data->pkt_len;
   if (config.what_to_count & COUNT_CLASS) {
-    if (!idata->pending_accumulators) {
-      Cursor->bytes_counter += data->cst.ba;
-      Cursor->packet_counter += data->cst.pa;
-      Cursor->flows_counter += data->cst.fa;
-      Cursor->ba = 0;
-      Cursor->pa = 0;
-      Cursor->fa = 0;
-    }
-    else {
-      Cursor->ba = data->cst.ba;
-      Cursor->pa = data->cst.pa;
-      Cursor->fa = data->cst.fa;
-    }
+    Cursor->bytes_counter += data->cst.ba;
+    Cursor->packet_counter += data->cst.pa;
+    Cursor->flows_counter += data->cst.fa;
+    Cursor->tentatives = data->cst.tentatives;
   }
-  Cursor->valid = TRUE;
+  Cursor->valid = SQL_CACHE_INUSE;
   Cursor->basetime = basetime;
+  Cursor->start_tag = idata->now;
   Cursor->lru_tag = idata->now;
   Cursor->signature = idata->hash;
   if (Cursor->chained) AddToLRUTail(Cursor); /* we cannot reuse not-malloc()'ed elements */
@@ -435,16 +463,10 @@ void sql_cache_insert(struct pkt_data *data, struct insert_data *idata)
   Cursor->flows_counter += data->flo_num;
   Cursor->bytes_counter += data->pkt_len;
   if (config.what_to_count & COUNT_CLASS) {
-    if (!idata->pending_accumulators) {
-      Cursor->bytes_counter += data->cst.ba;
-      Cursor->packet_counter += data->cst.pa;
-      Cursor->flows_counter += data->cst.fa;
-    }
-    else {
-      Cursor->ba += data->cst.ba;
-      Cursor->pa += data->cst.pa;
-      Cursor->fa += data->cst.fa;
-    }
+    Cursor->bytes_counter += data->cst.ba;
+    Cursor->packet_counter += data->cst.pa;
+    Cursor->flows_counter += data->cst.fa;
+    Cursor->tentatives = data->cst.tentatives;
   }
   return;
 
@@ -457,15 +479,15 @@ void sql_cache_insert(struct pkt_data *data, struct insert_data *idata)
     signal(SIGHUP, SIG_IGN);
     pm_setproctitle("%s [%s]", "SQL Plugin -- DB Writer (urgent)", config.name);
 
-    memset(&p, 0, sizeof(p));
-    memset(&b, 0, sizeof(b));
-
     (*sqlfunc_cbr.connect)(&p, config.sql_host);
     (*sqlfunc_cbr.purge)(queries_queue, qq_ptr, idata);
     (*sqlfunc_cbr.close)(&bed);
     exit(0);
   default: /* Parent */
-    qq_ptr = sql_cache_flush(queries_queue, qq_ptr);
+    sql_cache_flush(queries_queue, qq_ptr, idata);
+    qq_ptr = pqq_ptr;
+    pqq_ptr = 0;
+    memcpy(queries_queue, pending_queries_queue, sizeof(queries_queue));
     break;
   }
   if (SafePtr) {
@@ -598,10 +620,15 @@ void sql_exit_gracefully(int signum)
   idata.dyn_table = glob_dyn_table;
   idata.new_basetime = glob_new_basetime;
   if (config.sql_backup_host || config.sql_recovery_logfile) idata.recover = TRUE;
+  if (config.what_to_count & COUNT_CLASS) config.sql_aggressive_classification = FALSE;
 
-  (*sqlfunc_cbr.connect)(&p, config.sql_host);
-  (*sqlfunc_cbr.purge)(queries_queue, qq_ptr, &idata);
-  (*sqlfunc_cbr.close)(&bed);
+  sql_cache_flush(queries_queue, qq_ptr, &idata);
+  if (sql_writers.flags != CHLD_ALERT) {
+    if (sql_writers.flags == CHLD_WARNING) sql_db_fail(&p);
+    (*sqlfunc_cbr.connect)(&p, config.sql_host);
+    (*sqlfunc_cbr.purge)(queries_queue, qq_ptr, &idata);
+    (*sqlfunc_cbr.close)(&bed);
+  }
 
   exit_plugin(0);
 }
@@ -610,6 +637,7 @@ int sql_evaluate_primitives(int primitive)
 {
   u_int32_t what_to_count = 0, fakes = 0;
   short int assume_custom_table = FALSE; 
+  char *insert_clause_start_ptr = insert_clause + strlen(insert_clause);
 
   /* SQL tables < v6 multiplex IP addresses and AS numbers on the same field, thus are
      unable to use both them for a same direction (ie. src, dst). Tables v6 break such
@@ -1067,16 +1095,23 @@ int sql_evaluate_primitives(int primitive)
     primitive++;
   }
 
+  strncat(copy_clause, insert_clause_start_ptr, SPACELEFT(copy_clause));
+
   return primitive;
 }
 
 int sql_query(struct BE_descs *bed, struct db_cache *elem, struct insert_data *idata)
 {
-  if (!bed->p->fail && (elem->valid > 0)) {
-    if ((*sqlfunc_cbr.op)(bed->p, elem, idata)); /* don't return */ 
-    else return FALSE;
+  if (!bed->p->fail && elem->valid == SQL_CACHE_COMMITTED) {
+    if ((*sqlfunc_cbr.op)(bed->p, elem, idata)); /* failed */
+    else {
+      idata->qn++;
+      return FALSE;
+    }
   }
 
+  if ( elem->valid == SQL_CACHE_ERROR || (bed->p->fail && !(elem->valid == SQL_CACHE_INUSE)) ) {
+
   if (config.sql_backup_host) {
     if (!bed->b->fail) {
       if (!bed->b->connected) {
@@ -1114,6 +1149,8 @@ int sql_query(struct BE_descs *bed, struct db_cache *elem, struct insert_data *i
     }
   }
 
+  }
+
   quit:
   return TRUE;
 }
@@ -1219,7 +1256,24 @@ void sql_invalidate_shadow_entries(struct db_cache *queue[], int *num)
 
   for (x = 0; x < *num; x++) {
     if (!queue[x]->bytes_counter && !queue[x]->packet_counter && !queue[x]->flows_counter)
-      queue[x]->valid = FALSE;
+      queue[x]->valid = SQL_CACHE_FREE;
   }
 }
 
+int sql_select_locking_style(char *lock)
+{
+  int i = 0, len = strlen(lock);
+
+  while (i < len) {
+    lock[i] = tolower(lock[i]);
+    i++;
+  }
+
+  if (!strcmp(lock, "table")) return PM_LOCK_EXCLUSIVE;
+  else if (!strcmp(lock, "row")) return PM_LOCK_ROW_EXCLUSIVE;
+
+  Log(LOG_WARNING, "WARN ( %s/%s ): sql_locking_style value '%s' is unknown. Ignored.\n", config.name, config.type, lock);
+
+  return PM_LOCK_EXCLUSIVE;
+}
+
