@@ -169,6 +169,17 @@ Inline void SQL_SetENV()
     count++;
   }
 
+  {
+    u_char *tmpptr;
+
+    strncat(envbuf.ptr, "SQL_MAX_WRITERS=", envbuf.end-envbuf.ptr);
+    tmpptr = envbuf.ptr + strlen(envbuf.ptr);
+    snprintf(tmpptr, envbuf.end-tmpptr, "%d", config.sql_max_writers);
+    ptrs[count] = envbuf.ptr;
+    envbuf.ptr += strlen(envbuf.ptr)+1;
+    count++;
+  }
+
   for (i = 0; i < count; i++)
     putenv(ptrs[i]);
 }
@@ -270,6 +281,17 @@ Inline void SQL_SetENV_child(const struct insert_data *idata)
     count++;
   }
 
+  {
+    u_char *tmpptr;
+
+    strncat(envbuf.ptr, "SQL_ACTIVE_WRITERS=", envbuf.end-envbuf.ptr);
+    tmpptr = envbuf.ptr + strlen(envbuf.ptr);
+    snprintf(tmpptr, envbuf.end-tmpptr, "%d", sql_writers.active);
+    ptrs[count] = envbuf.ptr;
+    envbuf.ptr += strlen(envbuf.ptr)+1;
+    count++;
+  }
+
   for (i = 0; i < count; i++)
     putenv(ptrs[i]);
 }
