@@ -327,6 +327,37 @@ time_t calc_monthly_timeslot(time_t t, int howmany, int op)
   return (final-base);
 }	
 
+FILE *open_logfile(char *filename)
+{
+  char timebuf[SRVBUFLEN];
+  FILE *file = NULL;
+  struct tm *tmnow;
+  time_t now;
+
+
+  file = fopen(filename, "a"); 
+  if (file) {
+    if (file_lock(fileno(file))) {
+      Log(LOG_ALERT, "ALERT: Unable to obtain lock for logfile '%s'.\n", filename);
+      file = NULL;
+    }
+  }
+  else {
+    Log(LOG_ERR, "ERROR: Unable to open logfile '%s'\n", filename);
+    file = NULL;
+  }
+
+  if (file) {
+    now = time(NULL);
+    tmnow = localtime(&now);
+    strftime(timebuf, SRVBUFLEN, "%Y-%m-%d %H:%M:%S" , tmnow);
+    fprintf(file, "\n\n=== Start logging: %s ===\n\n", timebuf); 
+    fflush(file);
+  }
+
+  return file;
+}
+
 void write_pid_file(char *filename)
 {
   FILE *file;
@@ -337,7 +368,7 @@ void write_pid_file(char *filename)
   file = fopen(filename,"w");
   if (file) {
     if (file_lock(fileno(file))) {
-      Log(LOG_ALERT, "ALERT: Unable to obtain lock of '%s'.\n", filename);
+      Log(LOG_ALERT, "ALERT: Unable to obtain lock for pidfile '%s'.\n", filename);
       return;
     }
     sprintf(pid, "%d\n", getpid());
@@ -347,7 +378,7 @@ void write_pid_file(char *filename)
     fclose(file);
   }
   else {
-    Log(LOG_ERR, "ERROR: Unable to open file '%s'\n", filename);
+    Log(LOG_ERR, "ERROR: Unable to open pidfile '%s'\n", filename);
     return;
   }
 }
@@ -620,7 +651,7 @@ int file_archive(const char *path, int rotations)
   }
 
   /* we should never reach this point */
-  Log(LOG_ALERT, "ALERT: No more logfile ( %s ) rotations allowed. Data is getting lost.\n", path);  
+  Log(LOG_ALERT, "ALERT: No more recovery logfile ( %s ) rotations allowed. Data is getting lost.\n", path);  
   return -1;
 }
 
@@ -714,3 +745,53 @@ void exit_plugin(int status)
 
   exit(status);
 }
+
+void reset_tag_status(struct packet_ptrs_vector *pptrsv)
+{
+  pptrsv->v4.tag = FALSE;
+  pptrsv->vlan4.tag = FALSE;
+  pptrsv->mpls4.tag = FALSE;
+  pptrsv->vlanmpls4.tag = FALSE;
+
+#if defined ENABLE_IPV6
+  pptrsv->v6.tag = FALSE;
+  pptrsv->vlan6.tag = FALSE;
+  pptrsv->mpls6.tag = FALSE;
+  pptrsv->vlanmpls6.tag = FALSE;
+#endif
+}
+
+void reset_shadow_status(struct packet_ptrs_vector *pptrsv)
+{
+  pptrsv->v4.shadow = FALSE;
+  pptrsv->vlan4.shadow = FALSE;
+  pptrsv->mpls4.shadow = FALSE;
+  pptrsv->vlanmpls4.shadow = FALSE;
+
+#if defined ENABLE_IPV6
+  pptrsv->v6.shadow = FALSE;
+  pptrsv->vlan6.shadow = FALSE;
+  pptrsv->mpls6.shadow = FALSE;
+  pptrsv->vlanmpls6.shadow = FALSE;
+#endif
+}
+
+void reset_tagdist_status(struct packet_ptrs_vector *pptrsv)
+{
+  pptrsv->v4.tag_dist = TRUE;
+  pptrsv->vlan4.tag_dist = TRUE;
+  pptrsv->mpls4.tag_dist = TRUE;
+  pptrsv->vlanmpls4.tag_dist = TRUE;
+
+#if defined ENABLE_IPV6
+  pptrsv->v6.tag_dist = TRUE;
+  pptrsv->vlan6.tag_dist = TRUE;
+  pptrsv->mpls6.tag_dist = TRUE;
+  pptrsv->vlanmpls6.tag_dist = TRUE;
+#endif
+}
+
+void set_shadow_status(struct packet_ptrs *pptrs)
+{
+  pptrs->shadow = TRUE;
+}
